@@ -3,7 +3,7 @@
  *
  * pidgin-sipe
  *
- * Copyright (C) 2010-2013 SIPE Project <http://sipe.sourceforge.net/>
+ * Copyright (C) 2010-2016 SIPE Project <http://sipe.sourceforge.net/>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -64,7 +64,6 @@ typedef enum {
 	SIPE_DEBUG_LEVEL_INFO,
 	SIPE_DEBUG_LEVEL_WARNING,
 	SIPE_DEBUG_LEVEL_ERROR,
-	SIPE_DEBUG_LEVEL_FATAL,
 }  sipe_debug_level;
 
 /**
@@ -97,8 +96,6 @@ void sipe_backend_debug(sipe_debug_level level,
 #define SIPE_DEBUG_WARNING_NOFORMAT(msg) sipe_backend_debug_literal(SIPE_DEBUG_LEVEL_WARNING, msg)
 #define SIPE_DEBUG_ERROR(fmt, ...)       sipe_backend_debug(SIPE_DEBUG_LEVEL_ERROR,   fmt, __VA_ARGS__)
 #define SIPE_DEBUG_ERROR_NOFORMAT(msg)   sipe_backend_debug_literal(SIPE_DEBUG_LEVEL_ERROR,   msg)
-#define SIPE_DEBUG_FATAL(fmt, ...)       sipe_backend_debug(SIPE_DEBUG_LEVEL_FATAL,   fmt, __VA_ARGS__)
-#define SIPE_DEBUG_FATAL_NOFORMAT(msg)   sipe_backend_debug_literal(SIPE_DEBUG_LEVEL_FATAL,   msg)
 
 /**
  * Check backend debugging status
@@ -233,6 +230,7 @@ gssize sipe_backend_ft_write(struct sipe_file_transfer *ft,
 			     const guchar *data,
 			     gsize size);
 
+void sipe_backend_ft_set_completed(struct sipe_file_transfer *ft);
 
 void sipe_backend_ft_cancel_local(struct sipe_file_transfer *ft);
 void sipe_backend_ft_cancel_remote(struct sipe_file_transfer *ft);
@@ -242,6 +240,18 @@ void sipe_backend_ft_incoming(struct sipe_core_public *sipe_public,
 			      const gchar *who,
 			      const gchar *file_name,
 			      gsize file_size);
+/**
+ * Allocates and initializes backend file transfer structure for sending a file.
+ *
+ * @param sipe_public (in) the handle representing the protocol instance
+ * @param ft (in) sipe core file transfer structure
+ * @param who (in) SIP URI of the file recipient
+ * @param file_name (in) filesystem path of the file being sent
+ */
+void sipe_backend_ft_outgoing(struct sipe_core_public *sipe_public,
+			      struct sipe_file_transfer *ft,
+			      const gchar *who,
+			      const gchar *file_name);
 /**
  * Begins file transfer with remote peer.
  *
@@ -310,6 +320,13 @@ gchar *sipe_backend_markup_strip_html(const gchar *html);
 
 /** MEDIA ********************************************************************/
 
+typedef enum {
+	/* This client is the one who invites other participant to the call. */
+	SIPE_MEDIA_CALL_INITIATOR = 1,
+	/* Don't show any user interface elements for the call. */
+	SIPE_MEDIA_CALL_NO_UI = 2
+} SipeMediaCallFlags;
+
 typedef enum {
 	SIPE_ICE_NO_ICE,
 	SIPE_ICE_DRAFT_6,
@@ -332,27 +349,52 @@ typedef enum {
 
 typedef enum {
 	SIPE_MEDIA_AUDIO,
-	SIPE_MEDIA_VIDEO
+	SIPE_MEDIA_VIDEO,
+	SIPE_MEDIA_APPLICATION
 } SipeMediaType;
 
 typedef enum {
+	SIPE_NETWORK_PROTOCOL_UDP,
 	SIPE_NETWORK_PROTOCOL_TCP_ACTIVE,
 	SIPE_NETWORK_PROTOCOL_TCP_PASSIVE,
-	SIPE_NETWORK_PROTOCOL_UDP
+	SIPE_NETWORK_PROTOCOL_TCP_SO,
 } SipeNetworkProtocol;
 
+typedef enum {
+	SIPE_ENCRYPTION_POLICY_REJECTED,
+	SIPE_ENCRYPTION_POLICY_OPTIONAL,
+	SIPE_ENCRYPTION_POLICY_REQUIRED,
+	SIPE_ENCRYPTION_POLICY_OBEY_SERVER
+} SipeEncryptionPolicy;
+
 struct sipe_media_call;
 struct sipe_backend_media;
 struct sipe_backend_codec;
 struct sipe_backend_candidate;
-struct sipe_backend_stream;
+struct sipe_backend_media_stream;
 struct sipe_backend_media_relays;
 
+struct sipe_media_stream {
+	struct sipe_backend_media_stream *backend_private;
+
+	struct sipe_media_call *call;
+	gchar *id;
+
+	void (*candidate_pairs_established_cb)(struct sipe_media_stream *);
+	void (*read_cb)(struct sipe_media_stream *);
+	void (*writable_cb)(struct sipe_media_stream *);
+	void (*mute_cb)(struct sipe_media_stream *, gboolean is_muted);
+};
+
 struct sipe_media_call {
 	struct sipe_backend_media *backend_private;
 
+	gchar *with;
+
 	void (*stream_initialized_cb)(struct sipe_media_call *,
-				      struct sipe_backend_stream *);
+				      struct sipe_media_stream *);
+	void (*stream_end_cb)(struct sipe_media_call *,
+			      struct sipe_media_stream *);
 	void (*media_end_cb)(struct sipe_media_call *);
 	void (*call_accept_cb)(struct sipe_media_call *, gboolean local);
 	void (*call_reject_cb)(struct sipe_media_call *, gboolean local);
@@ -373,7 +415,7 @@ struct sipe_media_relay {
 struct sipe_backend_media *sipe_backend_media_new(struct sipe_core_public *sipe_public,
 						  struct sipe_media_call *call,
 						  const gchar *participant,
-						  gboolean initiator);
+						  SipeMediaCallFlags flags);
 void sipe_backend_media_free(struct sipe_backend_media *media);
 
 void sipe_backend_media_set_cname(struct sipe_backend_media *media, gchar *cname);
@@ -383,40 +425,47 @@ struct sipe_backend_media_relays * sipe_backend_media_relays_convert(GSList *med
 								     gchar *password);
 void sipe_backend_media_relays_free(struct sipe_backend_media_relays *media_relays);
 
-struct sipe_backend_stream *sipe_backend_media_add_stream(struct sipe_backend_media *media,
+struct sipe_backend_media_stream *sipe_backend_media_add_stream(struct sipe_media_call *media,
 							  const gchar *id,
 							  const gchar *participant,
 							  SipeMediaType type,
 							  SipeIceVersion ice_version,
 							  gboolean initiator,
-							  struct sipe_backend_media_relays *media_relays);
-void sipe_backend_media_remove_stream(struct sipe_backend_media *media,
-				      struct sipe_backend_stream *stream);
-GSList *sipe_backend_media_get_streams(struct sipe_backend_media *media);
-struct sipe_backend_stream *sipe_backend_media_get_stream_by_id(struct sipe_backend_media *media,
-								const gchar *id);
-void sipe_backend_media_add_remote_candidates(struct sipe_backend_media *media,
-					      struct sipe_backend_stream *stream,
+							  struct sipe_backend_media_relays *media_relays,
+							  guint min_port, guint max_port);
+void sipe_backend_media_add_remote_candidates(struct sipe_media_call *media,
+					      struct sipe_media_stream *stream,
 					      GList *candidates);
-gboolean sipe_backend_media_is_initiator(struct sipe_backend_media *media,
-					 struct sipe_backend_stream *stream);
+gboolean sipe_backend_media_is_initiator(struct sipe_media_call *media,
+					 struct sipe_media_stream *stream);
 gboolean sipe_backend_media_accepted(struct sipe_backend_media *media);
-gboolean sipe_backend_stream_initialized(struct sipe_backend_media *media,
-					 struct sipe_backend_stream *stream);
-GList *sipe_backend_media_get_active_local_candidates(struct sipe_backend_media *media,
-						      struct sipe_backend_stream *stream);
-GList *sipe_backend_media_get_active_remote_candidates(struct sipe_backend_media *media,
-						       struct sipe_backend_stream *stream);
+gboolean sipe_backend_stream_initialized(struct sipe_media_call *media,
+					 struct sipe_media_stream *stream);
+void sipe_backend_media_set_encryption_keys(struct sipe_media_call *media,
+					    struct sipe_media_stream *stream,
+					    const guchar *encryption_key,
+					    const guchar *decryption_key);
 
 /* Stream handling */
-const gchar *sipe_backend_stream_get_id(struct sipe_backend_stream *stream);
-void sipe_backend_stream_hold(struct sipe_backend_media *media,
-			      struct sipe_backend_stream *stream,
+void sipe_backend_stream_hold(struct sipe_media_call *media,
+			      struct sipe_media_stream *stream,
 			      gboolean local);
-void sipe_backend_stream_unhold(struct sipe_backend_media *media,
-				struct sipe_backend_stream *stream,
+void sipe_backend_stream_unhold(struct sipe_media_call *media,
+				struct sipe_media_stream *stream,
 				gboolean local);
-gboolean sipe_backend_stream_is_held(struct sipe_backend_stream *stream);
+gboolean sipe_backend_stream_is_held(struct sipe_media_stream *stream);
+
+GList *sipe_backend_media_stream_get_active_local_candidates(struct sipe_media_stream *stream);
+GList *sipe_backend_media_stream_get_active_remote_candidates(struct sipe_media_stream *stream);
+
+gssize sipe_backend_media_stream_read(struct sipe_media_stream *stream,
+				     guint8 *buffer, gsize len);
+gssize sipe_backend_media_stream_write(struct sipe_media_stream *stream,
+				       guint8 *buffer, gsize len);
+
+void sipe_backend_media_stream_end(struct sipe_media_call *media,
+				   struct sipe_media_stream *stream);
+void sipe_backend_media_stream_free(struct sipe_backend_media_stream *stream);
 
 /* Codec handling */
 struct sipe_backend_codec *sipe_backend_codec_new(int id,
@@ -433,11 +482,11 @@ void sipe_backend_codec_add_optional_parameter(struct sipe_backend_codec *codec,
 					       const gchar *name,
 					       const gchar *value);
 GList *sipe_backend_codec_get_optional_parameters(struct sipe_backend_codec *codec);
-gboolean sipe_backend_set_remote_codecs(struct sipe_backend_media *media,
-					struct sipe_backend_stream *stream,
+gboolean sipe_backend_set_remote_codecs(struct sipe_media_call *media,
+					struct sipe_media_stream *stream,
 					GList *codecs);
-GList* sipe_backend_get_local_codecs(struct sipe_backend_media *media,
-				     struct sipe_backend_stream *stream);
+GList* sipe_backend_get_local_codecs(struct sipe_media_call *media,
+				     struct sipe_media_stream *stream);
 
 /* Candidate handling */
 struct sipe_backend_candidate * sipe_backend_candidate_new(const gchar *foundation,
@@ -475,8 +524,8 @@ void sipe_backend_candidate_set_priority(struct sipe_backend_candidate *candidat
 SipeComponentType sipe_backend_candidate_get_component_type(struct sipe_backend_candidate *candidate);
 SipeCandidateType sipe_backend_candidate_get_type(struct sipe_backend_candidate *candidate);
 SipeNetworkProtocol sipe_backend_candidate_get_protocol(struct sipe_backend_candidate *candidate);
-GList* sipe_backend_get_local_candidates(struct sipe_backend_media *media,
-					 struct sipe_backend_stream *stream);
+GList* sipe_backend_get_local_candidates(struct sipe_media_call *media,
+					 struct sipe_media_stream *stream);
 void sipe_backend_media_accept(struct sipe_backend_media *media, gboolean local);
 void sipe_backend_media_hangup(struct sipe_backend_media *media, gboolean local);
 void sipe_backend_media_reject(struct sipe_backend_media *media, gboolean local);
@@ -498,6 +547,7 @@ sipe_backend_network_listen_range(unsigned short port_min,
 				  gpointer data);
 void sipe_backend_network_listen_cancel(struct sipe_backend_listendata *ldata);
 
+struct sipe_backend_fd * sipe_backend_fd_from_int(int fd);
 gboolean sipe_backend_fd_is_valid(struct sipe_backend_fd *fd);
 void sipe_backend_fd_free(struct sipe_backend_fd *fd);
 
@@ -571,6 +621,16 @@ guint sipe_backend_status(struct sipe_core_public *sipe_public);
 gboolean sipe_backend_status_changed(struct sipe_core_public *sipe_public,
 				     guint activity,
 				     const gchar *message);
+
+/**
+ * Update user client with new status and note received from server
+ *
+ * NOTE: this must *NOT* trigger a call to @c sipe_core_status_set()!
+ *
+ * @param sipe_public   The handle representing the protocol instance
+ * @param activity      New activity
+ * @param message       New note text
+ */
 void sipe_backend_status_and_note(struct sipe_core_public *sipe_public,
 				  guint activity,
 				  const gchar *message);
@@ -1013,6 +1073,8 @@ struct sipe_backend_buddy_menu *sipe_backend_buddy_sub_menu_add(struct sipe_core
 								const gchar *label,
 								struct sipe_backend_buddy_menu *sub);
 
+SipeEncryptionPolicy sipe_backend_media_get_encryption_policy(struct sipe_core_public *sipe_public);
+
 #ifdef __cplusplus
 }
 #endif
