@@ -3,7 +3,7 @@
  *
  * pidgin-sipe
  *
- * Copyright (C) 2013 SIPE Project <http://sipe.sourceforge.net/>
+ * Copyright (C) 2013-2015 SIPE Project <http://sipe.sourceforge.net/>
  * Copyright (C) 2010 Jakub Adam <jakub.adam@ktknet.cz>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,6 +28,7 @@
 #include <glib.h>
 
 #include "sipe-backend.h"
+#include "sipe-core.h"
 #include "sdpmsg.h"
 #include "sipe-utils.h"
 
@@ -56,16 +57,33 @@ parse_attributes(struct sdpmsg *smsg, gchar *msg) {
 	while (*ptr != NULL) {
 		if (g_str_has_prefix(*ptr, "o=")) {
 			gchar **parts = g_strsplit(*ptr + 2, " ", 6);
+			if (g_strv_length(parts) != 6) {
+				g_strfreev(parts);
+				g_strfreev(lines);
+				return FALSE;
+			}
+
 			smsg->ip = g_strdup(parts[5]);
 			g_strfreev(parts);
 		} else if (g_str_has_prefix(*ptr, "m=")) {
-			gchar **parts = g_strsplit(*ptr + 2, " ", 3);
-			struct sdpmedia *media = g_new0(struct sdpmedia, 1);
+			gchar **parts;
+			struct sdpmedia *media;
+
+			parts = g_strsplit(*ptr + 2, " ", 3);
+			if (g_strv_length(parts) < 3) {
+				g_strfreev(parts);
+				g_strfreev(lines);
+				return FALSE;
+			}
+
+			media = g_new0(struct sdpmedia, 1);
 
 			smsg->media = g_slist_append(smsg->media, media);
 
 			media->name = g_strdup(parts[0]);
 			media->port = atoi(parts[1]);
+			media->encryption_active =
+					g_strstr_len(parts[2], -1, "/SAVP") != NULL;
 
 			g_strfreev(parts);
 
@@ -90,7 +108,6 @@ parse_attributes(struct sdpmsg *smsg, gchar *msg) {
 }
 
 static struct sdpcandidate * sdpcandidate_copy(struct sdpcandidate *candidate);
-static void sdpcandidate_free(struct sdpcandidate *candidate);
 
 static SipeComponentType
 parse_component(const gchar *str)
@@ -124,10 +141,16 @@ base64_pad(const gchar* str)
 		return g_strdup(str);
 }
 
-static GSList *
-parse_append_candidate_draft_6(gchar **tokens, GSList *candidates)
+static gboolean
+parse_append_candidate_draft_6(gchar **tokens, GSList **candidates)
 {
-	struct sdpcandidate *candidate = g_new0(struct sdpcandidate, 1);
+	struct sdpcandidate *candidate;
+
+	if (g_strv_length(tokens) < 7 || strlen(tokens[4]) < 3) {
+		return FALSE;
+	}
+
+	candidate = g_new0(struct sdpcandidate, 1);
 
 	candidate->username = base64_pad(tokens[0]);
 	candidate->component = parse_component(tokens[1]);
@@ -139,88 +162,108 @@ parse_append_candidate_draft_6(gchar **tokens, GSList *candidates)
 		candidate->protocol = SIPE_NETWORK_PROTOCOL_TCP_ACTIVE;
 	else {
 		sdpcandidate_free(candidate);
-		return candidates;
+		return FALSE;
 	}
 
 	candidate->priority = atoi(tokens[4] + 2);
 	candidate->ip = g_strdup(tokens[5]);
 	candidate->port = atoi(tokens[6]);
 
-	candidates = g_slist_append(candidates, candidate);
+	*candidates = g_slist_append(*candidates, candidate);
 
 	// draft 6 candidates are both active and passive
 	if (candidate->protocol == SIPE_NETWORK_PROTOCOL_TCP_ACTIVE) {
 		candidate = sdpcandidate_copy(candidate);
 		candidate->protocol = SIPE_NETWORK_PROTOCOL_TCP_PASSIVE;
-		candidates = g_slist_append(candidates, candidate);
+		*candidates = g_slist_append(*candidates, candidate);
 	}
 
-	return candidates;
+	return TRUE;
 }
 
-static GSList *
-parse_append_candidate_rfc_5245(gchar **tokens, GSList *candidates)
+static gboolean
+parse_append_candidate_rfc_5245(gchar **tokens, GSList **candidates)
 {
-	struct sdpcandidate *candidate = g_new0(struct sdpcandidate, 1);
+	struct sdpcandidate *candidate;
 
+	if (g_strv_length(tokens) < 8) {
+		return FALSE;
+	}
+
+	candidate = g_new0(struct sdpcandidate, 1);
 	candidate->foundation = g_strdup(tokens[0]);
 	candidate->component = parse_component(tokens[1]);
 
-	if (sipe_strequal(tokens[2], "UDP"))
+	if (sipe_strcase_equal(tokens[2], "UDP"))
 		candidate->protocol = SIPE_NETWORK_PROTOCOL_UDP;
-	else if (sipe_strequal(tokens[2], "TCP-ACT"))
+	else if (sipe_strcase_equal(tokens[2], "TCP-ACT"))
 		candidate->protocol = SIPE_NETWORK_PROTOCOL_TCP_ACTIVE;
-	else if (sipe_strequal(tokens[2], "TCP-PASS"))
+	else if (sipe_strcase_equal(tokens[2], "TCP-PASS"))
 		candidate->protocol = SIPE_NETWORK_PROTOCOL_TCP_PASSIVE;
 	else {
 		sdpcandidate_free(candidate);
-		return candidates;
+		return FALSE;
 	}
 
 	candidate->priority = atoi(tokens[3]);
 	candidate->ip = g_strdup(tokens[4]);
 	candidate->port = atoi(tokens[5]);
 
-	if (sipe_strequal(tokens[7], "host"))
+	if (sipe_strcase_equal(tokens[7], "host"))
 		candidate->type = SIPE_CANDIDATE_TYPE_HOST;
-	else if (sipe_strequal(tokens[7], "relay"))
+	else if (sipe_strcase_equal(tokens[7], "relay"))
 		candidate->type = SIPE_CANDIDATE_TYPE_RELAY;
-	else if (sipe_strequal(tokens[7], "srflx"))
+	else if (sipe_strcase_equal(tokens[7], "srflx"))
 		candidate->type = SIPE_CANDIDATE_TYPE_SRFLX;
-	else if (sipe_strequal(tokens[7], "prflx"))
+	else if (sipe_strcase_equal(tokens[7], "prflx"))
 		candidate->type = SIPE_CANDIDATE_TYPE_PRFLX;
 	else {
 		sdpcandidate_free(candidate);
-		return candidates;
+		return FALSE;
 	}
 
-	return g_slist_append(candidates, candidate);
+	*candidates = g_slist_append(*candidates, candidate);
+
+	return TRUE;
 }
 
-static GSList *
-parse_candidates(GSList *attrs, SipeIceVersion *ice_version)
+static gboolean
+parse_candidates(GSList *attrs, SipeIceVersion *ice_version, GSList **candidates)
 {
-	GSList *candidates = NULL;
 	const gchar *attr;
 	int i = 0;
 
+	g_return_val_if_fail(*candidates == NULL, FALSE);
+
 	while ((attr = sipe_utils_nameval_find_instance(attrs, "candidate", i++))) {
 		gchar **tokens = g_strsplit_set(attr, " ", 0);
+		gboolean parsed_ok;
+
+		if (g_strv_length(tokens) < 7) {
+			g_strfreev(tokens);
+			return FALSE;
+		}
 
 		if (sipe_strequal(tokens[6], "typ")) {
-			candidates = parse_append_candidate_rfc_5245(tokens, candidates);
-			if (candidates)
+			parsed_ok = parse_append_candidate_rfc_5245(tokens,
+								    candidates);
+			if (*candidates)
 				*ice_version = SIPE_ICE_RFC_5245;
 		} else {
-			candidates = parse_append_candidate_draft_6(tokens, candidates);
-			if (candidates)
+			parsed_ok = parse_append_candidate_draft_6(tokens,
+								   candidates);
+			if (*candidates)
 				*ice_version = SIPE_ICE_DRAFT_6;
 		}
 
 		g_strfreev(tokens);
+
+		if (!parsed_ok) {
+			return FALSE;
+		}
 	}
 
-	if (!candidates)
+	if (!(*candidates))
 		*ice_version = SIPE_ICE_NO_ICE;
 
 	if (*ice_version == SIPE_ICE_RFC_5245) {
@@ -229,7 +272,7 @@ parse_candidates(GSList *attrs, SipeIceVersion *ice_version)
 
 		if (username && password) {
 			GSList *i;
-			for (i = candidates; i; i = i->next) {
+			for (i = *candidates; i; i = i->next) {
 				struct sdpcandidate *c = i->data;
 				c->username = g_strdup(username);
 				c->password = g_strdup(password);
@@ -237,7 +280,7 @@ parse_candidates(GSList *attrs, SipeIceVersion *ice_version)
 		}
 	}
 
-	return candidates;
+	return TRUE;
 }
 
 static GSList *
@@ -269,50 +312,113 @@ create_legacy_candidates(gchar *ip, guint16 port)
 	return candidates;
 }
 
-static GSList *
-parse_codecs(GSList *attrs, SipeMediaType type)
+static gboolean
+parse_codec_parameters(GSList *attrs, struct sdpcodec *codec)
+{
+	const gchar* params;
+	int i = 0;
+
+	while((params = sipe_utils_nameval_find_instance(attrs, "fmtp", i++))) {
+		gchar **tokens;
+		gchar **param;
+
+		tokens = g_strsplit(params, " ", 0);
+		if (g_strv_length(tokens) < 1) {
+			g_strfreev(tokens);
+			return FALSE;
+		}
+
+		if (atoi(tokens[0]) != codec->id) {
+			g_strfreev(tokens);
+			continue;
+		}
+
+		for (param = tokens + 1; *param; ++param) {
+			gchar **nameval = g_strsplit(*param, "=", 2);
+
+			if (g_strv_length(nameval) != 2) {
+				g_strfreev(nameval);
+				continue;
+			}
+
+			codec->parameters =
+					sipe_utils_nameval_add(codec->parameters,
+							       nameval[0],
+							       nameval[1]);
+
+			g_strfreev(nameval);
+		}
+
+		g_strfreev(tokens);
+	}
+
+	return TRUE;
+}
+
+
+static gboolean
+parse_codecs(GSList *attrs, SipeMediaType type, GSList **codecs)
 {
 	int i = 0;
 	const gchar *attr;
-	GSList *codecs = NULL;
 
 	while ((attr = sipe_utils_nameval_find_instance(attrs, "rtpmap", i++))) {
-		struct sdpcodec *codec = g_new0(struct sdpcodec, 1);
-		gchar **tokens = g_strsplit_set(attr, " /", 3);
+		struct sdpcodec *codec;
+		gchar **tokens;
 
-		int j = 0;
-		const gchar* params;
+		tokens = g_strsplit_set(attr, " /", 3);
+		if (g_strv_length(tokens) != 3) {
+			g_strfreev(tokens);
+			return FALSE;
+		}
 
+		codec = g_new0(struct sdpcodec, 1);
 		codec->id = atoi(tokens[0]);
 		codec->name = g_strdup(tokens[1]);
 		codec->clock_rate = atoi(tokens[2]);
 		codec->type = type;
 
-		// TODO: more secure and effective implementation
-		while((params = sipe_utils_nameval_find_instance(attrs, "fmtp", j++))) {
-			gchar **tokens = g_strsplit_set(params, " ", 0);
-			gchar **next = tokens + 1;
+		g_strfreev(tokens);
 
-			if (atoi(tokens[0]) == codec->id) {
-				while (*next) {
-					gchar name[50];
-					gchar value[50];
-
-					if (sscanf(*next, "%[a-zA-Z0-9]=%s", name, value) == 2)
-						codec->parameters = sipe_utils_nameval_add(codec->parameters, name, value);
-
-					++next;
-				}
-			}
-
-			g_strfreev(tokens);
+		if (!parse_codec_parameters(attrs, codec)) {
+			sdpcodec_free(codec);
+			return FALSE;
 		}
 
-		codecs = g_slist_append(codecs, codec);
-		g_strfreev(tokens);
+		*codecs = g_slist_append(*codecs, codec);
 	}
 
-	return codecs;
+	return TRUE;
+}
+
+static void
+parse_encryption_key(GSList *attrs, guchar **key, int *key_id)
+{
+	int i = 0;
+	const gchar *attr;
+
+	while ((attr = sipe_utils_nameval_find_instance(attrs, "crypto", i++))) {
+		gchar **tokens = g_strsplit_set(attr, " :|", 6);
+
+		if (tokens[0] && tokens[1] && tokens[2] && tokens[3] && tokens[4] &&
+		    sipe_strcase_equal(tokens[1], "AES_CM_128_HMAC_SHA1_80") &&
+		    sipe_strequal(tokens[2], "inline") &&
+		    !tokens[5]) {
+			gsize key_len;
+			*key = g_base64_decode(tokens[3], &key_len);
+			if (key_len != SIPE_SRTP_KEY_LEN) {
+				g_free(*key);
+				*key = NULL;
+			}
+			*key_id = atoi(tokens[0]);
+		}
+
+		g_strfreev(tokens);
+
+		if (*key) {
+			break;
+		}
+	}
 }
 
 struct sdpmsg *
@@ -330,8 +436,11 @@ sdpmsg_parse_msg(gchar *msg)
 		struct sdpmedia *media = i->data;
 		SipeMediaType type;
 
-		media->candidates = parse_candidates(media->attributes,
-						     &smsg->ice_version);
+		if (!parse_candidates(media->attributes, &smsg->ice_version,
+				      &media->candidates)) {
+			sdpmsg_free(smsg);
+			return NULL;
+		}
 
 		if (!media->candidates && media->port != 0) {
 			// No a=candidate in SDP message, this seems to be MSOC 2005
@@ -342,13 +451,21 @@ sdpmsg_parse_msg(gchar *msg)
 			type = SIPE_MEDIA_AUDIO;
 		else if (sipe_strequal(media->name, "video"))
 			type = SIPE_MEDIA_VIDEO;
+		else if (sipe_strequal(media->name, "data"))
+			type = SIPE_MEDIA_APPLICATION;
 		else {
 			// Unknown media type
 			sdpmsg_free(smsg);
 			return NULL;
 		}
 
-		media->codecs = parse_codecs(media->attributes, type);
+		if (!parse_codecs(media->attributes, type, &media->codecs)) {
+			sdpmsg_free(smsg);
+			return NULL;
+		}
+
+		parse_encryption_key(media->attributes, &media->encryption_key,
+				&media->encryption_key_id);
 	}
 
 	return smsg;
@@ -466,15 +583,9 @@ candidates_to_string(GSList *candidates, SipeIceVersion ice_version)
 					break;
 				case SIPE_CANDIDATE_TYPE_RELAY:
 					type = "relay";
-					related = g_strdup_printf("raddr %s rport %d ",
-								  c->base_ip,
-								  c->base_port);
 					break;
 				case SIPE_CANDIDATE_TYPE_SRFLX:
 					type = "srflx";
-					related = g_strdup_printf("raddr %s rport %d",
-								  c->base_ip,
-								  c->base_port);
 					break;
 				case SIPE_CANDIDATE_TYPE_PRFLX:
 					type = "prflx";
@@ -485,6 +596,18 @@ candidates_to_string(GSList *candidates, SipeIceVersion ice_version)
 					break;
 			}
 
+			switch (c->type) {
+				case SIPE_CANDIDATE_TYPE_RELAY:
+				case SIPE_CANDIDATE_TYPE_SRFLX:
+				case SIPE_CANDIDATE_TYPE_PRFLX:
+					related = g_strdup_printf("raddr %s rport %d",
+								  c->base_ip,
+								  c->base_port);
+					break;
+				default:
+					break;
+			}
+
 			g_string_append_printf(result,
 					       "a=candidate:%s %u %s %u %s %d typ %s %s\r\n",
 					       c->foundation,
@@ -607,6 +730,8 @@ media_to_string(const struct sdpmsg *msg, const struct sdpmedia *media)
 {
 	gchar *media_str;
 
+	gchar *transport_profile = NULL;
+
 	gchar *media_conninfo = NULL;
 
 	gchar *codecs_str = NULL;
@@ -615,11 +740,12 @@ media_to_string(const struct sdpmsg *msg, const struct sdpmedia *media)
 	gchar *candidates_str = NULL;
 	gchar *remote_candidates_str = NULL;
 
-	gchar *tcp_setup_str = NULL;
 	gchar *attributes_str = NULL;
 	gchar *credentials = NULL;
 
-	gboolean uses_tcp_transport = FALSE;
+	gchar *crypto = NULL;
+
+	gboolean uses_tcp_transport = TRUE;
 
 	if (media->port != 0) {
 		if (!sipe_strequal(msg->ip, media->ip)) {
@@ -635,12 +761,16 @@ media_to_string(const struct sdpmsg *msg, const struct sdpmedia *media)
 			struct sdpcandidate *c = media->remote_candidates->data;
 			uses_tcp_transport =
 				c->protocol == SIPE_NETWORK_PROTOCOL_TCP_ACTIVE ||
-				c->protocol == SIPE_NETWORK_PROTOCOL_TCP_PASSIVE;
-			if (uses_tcp_transport) {
-				tcp_setup_str = g_strdup_printf(
-					"a=connection:existing\r\n"
-					"a=setup:%s\r\n",
-					(c->protocol == SIPE_NETWORK_PROTOCOL_TCP_ACTIVE) ? "passive" : "active");
+				c->protocol == SIPE_NETWORK_PROTOCOL_TCP_PASSIVE ||
+				c->protocol == SIPE_NETWORK_PROTOCOL_TCP_SO;
+		} else {
+			GSList *candidates = media->candidates;
+			for (; candidates; candidates = candidates->next) {
+				struct sdpcandidate *c = candidates->data;
+				if (c->protocol == SIPE_NETWORK_PROTOCOL_UDP) {
+					uses_tcp_transport = FALSE;
+					break;
+				}
 			}
 		}
 
@@ -654,9 +784,20 @@ media_to_string(const struct sdpmsg *msg, const struct sdpmedia *media)
 						      c->username,
 						      c->password);
 		}
+
+		if (media->encryption_key) {
+			gchar *key_encoded = g_base64_encode(media->encryption_key, SIPE_SRTP_KEY_LEN);
+			crypto = g_strdup_printf("a=crypto:%d AES_CM_128_HMAC_SHA1_80 inline:%s|2^31\r\n",
+					media->encryption_key_id, key_encoded);
+			g_free(key_encoded);
+		}
 	}
 
-	media_str = g_strdup_printf("m=%s %d %sRTP/AVP%s\r\n"
+	transport_profile = g_strdup_printf("%sRTP/%sAVP",
+					    uses_tcp_transport ? "TCP/" : "",
+					    media->encryption_active ? "S" : "");
+
+	media_str = g_strdup_printf("m=%s %d %s%s\r\n"
 				    "%s"
 				    "%s"
 				    "%s"
@@ -664,23 +805,24 @@ media_to_string(const struct sdpmsg *msg, const struct sdpmedia *media)
 				    "%s"
 				    "%s"
 				    "%s",
-				    media->name, media->port, uses_tcp_transport ? "TCP/" : "", codec_ids_str,
+				    media->name, media->port, transport_profile, codec_ids_str,
 				    media_conninfo ? media_conninfo : "",
 				    candidates_str ? candidates_str : "",
+				    crypto ? crypto : "",
 				    remote_candidates_str ? remote_candidates_str : "",
-				    tcp_setup_str ? tcp_setup_str : "",
 				    codecs_str ? codecs_str : "",
 				    attributes_str ? attributes_str : "",
 				    credentials ? credentials : "");
 
+	g_free(transport_profile);
 	g_free(media_conninfo);
 	g_free(codecs_str);
 	g_free(codec_ids_str);
 	g_free(candidates_str);
 	g_free(remote_candidates_str);
-	g_free(tcp_setup_str);
 	g_free(attributes_str);
 	g_free(credentials);
+	g_free(crypto);
 
 	return media_str;
 }
@@ -734,7 +876,7 @@ sdpcandidate_copy(struct sdpcandidate *candidate)
 		return NULL;
 }
 
-static void
+void
 sdpcandidate_free(struct sdpcandidate *candidate)
 {
 	if (candidate) {
@@ -773,6 +915,8 @@ sdpmedia_free(struct sdpmedia *media)
 		sipe_utils_slist_free_full(media->remote_candidates,
 				  (GDestroyNotify) sdpcandidate_free);
 
+		g_free(media->encryption_key);
+
 		g_free(media);
 	}
 }
