@@ -3,7 +3,7 @@
  *
  * pidgin-sipe
  *
- * Copyright (C) 2010-2014 SIPE Project <http://sipe.sourceforge.net/>
+ * Copyright (C) 2010-2016 SIPE Project <http://sipe.sourceforge.net/>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,6 +18,9 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ * GetUserPhoto operation
+ *  <http://msdn.microsoft.com/en-us/library/office/jj900502.aspx>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -42,6 +45,7 @@
 #include "sipe-conf.h"
 #include "sipe-core.h"
 #include "sipe-core-private.h"
+#include "sipe-digest.h"
 #include "sipe-group.h"
 #include "sipe-http.h"
 #include "sipe-im.h"
@@ -542,7 +546,8 @@ gchar *sipe_core_buddy_status(struct sipe_core_public *sipe_public,
 		g_string_append(status, sbuddy->note);
 	}
 
-	return(g_string_free(status, FALSE));
+	/* return NULL instead of empty status text */
+	return(g_string_free(status, status->len ? FALSE : TRUE));
 }
 
 gchar *sipe_buddy_get_alias(struct sipe_core_private *sipe_private,
@@ -912,9 +917,14 @@ struct ms_dlx_data {
 				struct ms_dlx_data *mdd);
 };
 
+static void free_search_rows(GSList *search_rows)
+{
+	sipe_utils_slist_free_full(search_rows, g_free);
+}
+
 static void ms_dlx_free(struct ms_dlx_data *mdd)
 {
-	sipe_utils_slist_free_full(mdd->search_rows, g_free);
+	free_search_rows(mdd->search_rows);
 	sipe_svc_session_close(mdd->session);
 	g_free(mdd->other);
 	g_free(mdd->wsse_security);
@@ -936,17 +946,31 @@ static gchar * prepare_buddy_search_query(GSList *query_rows, gboolean use_dlx)
 	while (query_rows) {
 		gchar *attr;
 		gchar *value;
+		gchar *tmp = NULL;
 
 		attr = query_rows->data;
 		query_rows = g_slist_next(query_rows);
 		value = query_rows->data;
 		query_rows = g_slist_next(query_rows);
 
-		if (!attr || !value)
+		if (!value)
 			break;
 
+		/*
+		 * Special value for SIP ID
+		 *
+		 * Active Directory seems only to be able to search for
+		 * SIP URIs. Make sure search string starts with "sip:".
+		 */
+		if (!attr) {
+			attr = "msRTCSIP-PrimaryUserAddress";
+			if (!use_dlx)
+				value = tmp = sip_uri(value);
+		}
+
 		attrs[i++] = g_markup_printf_escaped(use_dlx ? DLX_SEARCH_ITEM : SIPE_SOAP_SEARCH_ROW,
 						     attr, value);
+		g_free(tmp);
 	}
 	attrs[i] = NULL;
 
@@ -971,28 +995,48 @@ static void ms_dlx_webticket(struct sipe_core_private *sipe_private,
 	struct ms_dlx_data *mdd = callback_data;
 
 	if (wsse_security) {
-		gchar *query = prepare_buddy_search_query(mdd->search_rows, TRUE);
+		guint length = g_slist_length(mdd->search_rows);
+		gchar *search;
 
 		SIPE_DEBUG_INFO("ms_dlx_webticket: got ticket for %s",
 				base_uri);
 
+		if (length > 0) {
+			/* complex search */
+			gchar *query = prepare_buddy_search_query(mdd->search_rows, TRUE);
+			search = g_strdup_printf("<ChangeSearch xmlns:q1=\"DistributionListExpander\" soapenc:arrayType=\"q1:AbEntryRequest.ChangeSearchQuery[%d]\">"
+						 " %s"
+						 "</ChangeSearch>",
+						 length / 2,
+						 query);
+			g_free(query);
+		} else {
+			/* simple search */
+			search = g_strdup_printf("<BasicSearch>"
+						 " <SearchList>c,company,displayName,givenName,mail,mailNickname,msRTCSIP-PrimaryUserAddress,sn</SearchList>"
+						 " <Value>%s</Value>"
+						 " <Verb>BeginsWith</Verb>"
+						 "</BasicSearch>",
+						 mdd->other);
+		}
+
 		if (sipe_svc_ab_entry_request(sipe_private,
 					      mdd->session,
 					      auth_uri,
 					      wsse_security,
-					      query,
-					      g_slist_length(mdd->search_rows) / 2,
+					      search,
 					      mdd->max_returns,
 					      mdd->callback,
 					      mdd)) {
 
 			/* keep webticket security token for potential further use */
+			g_free(mdd->wsse_security);
 			mdd->wsse_security = g_strdup(wsse_security);
 
 			/* callback data passed down the line */
 			mdd = NULL;
 		}
-		g_free(query);
+		g_free(search);
 
 	} else {
 		/* no ticket: this will show the minmum information */
@@ -1019,10 +1063,10 @@ static void ms_dlx_webticket_request(struct sipe_core_private *sipe_private,
 	}
 }
 
-static void search_contacts_finalize(struct sipe_core_private *sipe_private,
-				     struct sipe_backend_search_results *results,
-				     guint match_count,
-				     gboolean more)
+void sipe_buddy_search_contacts_finalize(struct sipe_core_private *sipe_private,
+					 struct sipe_backend_search_results *results,
+					 guint match_count,
+					 gboolean more)
 {
 	gchar *secondary = g_strdup_printf(
 		dngettext(PACKAGE_NAME,
@@ -1056,12 +1100,28 @@ static void search_ab_entry_response(struct sipe_core_private *sipe_private,
 		/* any matches? */
 		node = sipe_xml_child(soap_body, "Body/SearchAbEntryResponse/SearchAbEntryResult/Items/AbEntry");
 		if (!node) {
-			SIPE_DEBUG_ERROR_NOFORMAT("search_ab_entry_response: no matches");
-			sipe_backend_search_failed(SIPE_CORE_PUBLIC,
-						   mdd->token,
-						   _("No contacts found"));
-			ms_dlx_free(mdd);
-			return;
+			/* try again with simple search, if possible */
+			if (mdd->other && mdd->search_rows) {
+				SIPE_DEBUG_INFO_NOFORMAT("search_ab_entry_response: no matches, retrying with simple search");
+
+				/* throw away original search query */
+				free_search_rows(mdd->search_rows);
+				mdd->search_rows = NULL;
+
+				ms_dlx_webticket_request(sipe_private, mdd);
+
+				/* callback data passed down the line */
+				return;
+
+			} else {
+				SIPE_DEBUG_ERROR_NOFORMAT("search_ab_entry_response: no matches");
+
+				sipe_backend_search_failed(SIPE_CORE_PUBLIC,
+							   mdd->token,
+							   _("No contacts found"));
+				ms_dlx_free(mdd);
+				return;
+			}
 		}
 
 		/* OK, we found something - show the results to the user */
@@ -1146,9 +1206,9 @@ static void search_ab_entry_response(struct sipe_core_private *sipe_private,
 			g_free(sip_uri);
 		}
 
-		search_contacts_finalize(sipe_private, results,
-					 g_hash_table_size(found),
-					 FALSE);
+		sipe_buddy_search_contacts_finalize(sipe_private, results,
+						    g_hash_table_size(found),
+						    FALSE);
 		g_hash_table_destroy(found);
 		ms_dlx_free(mdd);
 
@@ -1234,25 +1294,29 @@ static gboolean process_search_contact_response(struct sipe_core_private *sipe_p
 		g_free(data);
 	}
 
-	search_contacts_finalize(sipe_private, results, match_count, more);
+	sipe_buddy_search_contacts_finalize(sipe_private, results, match_count, more);
 	sipe_xml_free(searchResults);
 
 	return(TRUE);
 }
 
 static void search_soap_request(struct sipe_core_private *sipe_private,
-				struct sipe_backend_search_token *token,
+				GDestroyNotify destroy,
+				void *data,
+				guint max,
+				SoapTransCallback callback,
 				GSList *search_rows)
 {
 	gchar *query = prepare_buddy_search_query(search_rows, FALSE);
 	struct transaction_payload *payload = g_new0(struct transaction_payload, 1);
 
-	payload->data = token;
+	payload->destroy = destroy;
+	payload->data    = data;
 
 	sip_soap_directory_search(sipe_private,
-				  100,
+				  max,
 				  query,
-				  process_search_contact_response,
+				  callback,
 				  payload);
 	g_free(query);
 }
@@ -1261,7 +1325,13 @@ static void search_ab_entry_failed(struct sipe_core_private *sipe_private,
 				   struct ms_dlx_data *mdd)
 {
 	/* error using [MS-DLX] server, retry using Active Directory */
-	search_soap_request(sipe_private, mdd->token, mdd->search_rows);
+	if (mdd->search_rows)
+		search_soap_request(sipe_private,
+				    NULL,
+				    mdd->token,
+				    100,
+				    process_search_contact_response,
+				    mdd->search_rows);
 	ms_dlx_free(mdd);
 }
 
@@ -1270,45 +1340,77 @@ void sipe_core_buddy_search(struct sipe_core_public *sipe_public,
 			    const gchar *given_name,
 			    const gchar *surname,
 			    const gchar *email,
+			    const gchar *sipid,
 			    const gchar *company,
 			    const gchar *country)
 {
-	GSList *query_rows = NULL;
+	struct sipe_core_private *sipe_private = SIPE_CORE_PRIVATE;
 
-#define ADD_QUERY_ROW(attr, val)                                               \
-	if (val) {                                                             \
-		query_rows = g_slist_append(query_rows, g_strdup(attr));       \
-		query_rows = g_slist_append(query_rows, g_strdup(val));        \
-	}
+	/* Lync 2013 or newer: use UCS if contacts are migrated */
+	if (SIPE_CORE_PRIVATE_FLAG_IS(LYNC2013) &&
+	    sipe_ucs_is_migrated(sipe_private)) {
 
-	ADD_QUERY_ROW("givenName", given_name);
-	ADD_QUERY_ROW("sn",        surname);
-	ADD_QUERY_ROW("mail",      email);
-	ADD_QUERY_ROW("company",   company);
-	ADD_QUERY_ROW("c",         country);
+		sipe_ucs_search(sipe_private,
+				token,
+				given_name,
+				surname,
+				email,
+				sipid,
+				company,
+				country);
 
-	if (query_rows) {
-		if (SIPE_CORE_PRIVATE->dlx_uri != NULL) {
-			struct ms_dlx_data *mdd = g_new0(struct ms_dlx_data, 1);
+	} else {
+		GSList *query_rows  = NULL;
+		guint count         = 0;
+		const gchar *simple = NULL;
 
-			mdd->search_rows     = query_rows;
-			mdd->max_returns     = 100;
-			mdd->callback        = search_ab_entry_response;
-			mdd->failed_callback = search_ab_entry_failed;
-			mdd->session         = sipe_svc_session_start();
-			mdd->token           = token;
-
-			ms_dlx_webticket_request(SIPE_CORE_PRIVATE, mdd);
-
-		} else {
-			/* no [MS-DLX] server, use Active Directory search instead */
-			search_soap_request(SIPE_CORE_PRIVATE, token, query_rows);
-			sipe_utils_slist_free_full(query_rows, g_free);
+#define ADD_QUERY_ROW(attr, val)                                                 \
+		if (val) {                                                       \
+			query_rows = g_slist_append(query_rows, g_strdup(attr)); \
+			query_rows = g_slist_append(query_rows, g_strdup(val));  \
+			simple = val;                                            \
+			count++;                                                 \
 		}
-	} else
-		sipe_backend_search_failed(sipe_public,
-					   token,
-					   _("Invalid contact search query"));
+
+		ADD_QUERY_ROW("givenName", given_name);
+		ADD_QUERY_ROW("sn",        surname);
+		ADD_QUERY_ROW("mail",      email);
+		/* prepare_buddy_search_query() interprets NULL as SIP ID */
+		ADD_QUERY_ROW(NULL,        sipid);
+		ADD_QUERY_ROW("company",   company);
+		ADD_QUERY_ROW("c",         country);
+
+		if (query_rows) {
+			if (sipe_private->dlx_uri != NULL) {
+				struct ms_dlx_data *mdd = g_new0(struct ms_dlx_data, 1);
+
+				mdd->search_rows     = query_rows;
+				/* user entered only one search string, remember that one */
+				if (count == 1)
+					mdd->other   = g_strdup(simple);
+				mdd->max_returns     = 100;
+				mdd->callback        = search_ab_entry_response;
+				mdd->failed_callback = search_ab_entry_failed;
+				mdd->session         = sipe_svc_session_start();
+				mdd->token           = token;
+
+				ms_dlx_webticket_request(sipe_private, mdd);
+
+			} else {
+				/* no [MS-DLX] server, use Active Directory search instead */
+				search_soap_request(sipe_private,
+						    NULL,
+						    token,
+						    100,
+						    process_search_contact_response,
+						    query_rows);
+				free_search_rows(query_rows);
+			}
+		} else
+			sipe_backend_search_failed(sipe_public,
+						   token,
+						   _("Invalid contact search query"));
+	}
 }
 
 static void get_info_finalize(struct sipe_core_private *sipe_private,
@@ -1618,34 +1720,33 @@ static void get_info_ab_entry_failed(struct sipe_core_private *sipe_private,
 				     struct ms_dlx_data *mdd)
 {
 	/* error using [MS-DLX] server, retry using Active Directory */
-	gchar *query = prepare_buddy_search_query(mdd->search_rows, FALSE);
-	struct transaction_payload *payload = g_new0(struct transaction_payload, 1);
-
-	payload->destroy = g_free;
-	payload->data = mdd->other;
+	search_soap_request(sipe_private,
+			    g_free,
+			    mdd->other,
+			    1,
+			    process_get_info_response,
+			    mdd->search_rows);
 	mdd->other = NULL;
-
-	sip_soap_directory_search(sipe_private,
-							  1,
-							  query,
-							  process_get_info_response,
-							  payload);
-
 	ms_dlx_free(mdd);
-	g_free(query);
+}
+
+static GSList *search_rows_for_uri(const gchar *uri)
+{
+	/* prepare_buddy_search_query() interprets NULL as SIP ID */
+	GSList *l = g_slist_append(NULL, NULL);
+	return(g_slist_append(l, g_strdup(uri)));
 }
 
 void sipe_core_buddy_get_info(struct sipe_core_public *sipe_public,
 			      const gchar *who)
 {
 	struct sipe_core_private *sipe_private = SIPE_CORE_PRIVATE;
+	GSList *search_rows = search_rows_for_uri(who);
 
 	if (sipe_private->dlx_uri) {
 		struct ms_dlx_data *mdd = g_new0(struct ms_dlx_data, 1);
 
-		mdd->search_rows = g_slist_append(mdd->search_rows, g_strdup("msRTCSIP-PrimaryUserAddress"));
-		mdd->search_rows = g_slist_append(mdd->search_rows, g_strdup(who));
-
+		mdd->search_rows     = search_rows;
 		mdd->other           = g_strdup(who);
 		mdd->max_returns     = 1;
 		mdd->callback        = get_info_ab_entry_response;
@@ -1656,23 +1757,13 @@ void sipe_core_buddy_get_info(struct sipe_core_public *sipe_public,
 
 	} else {
 		/* no [MS-DLX] server, use Active Directory search instead */
-		gchar *row = g_markup_printf_escaped(SIPE_SOAP_SEARCH_ROW,
-						     "msRTCSIP-PrimaryUserAddress",
-						     who);
-		struct transaction_payload *payload = g_new0(struct transaction_payload, 1);
-
-		SIPE_DEBUG_INFO("sipe_core_buddy_get_info: row: %s",
-				row ? row : "");
-
-		payload->destroy = g_free;
-		payload->data = g_strdup(who);
-
-		sip_soap_directory_search(sipe_private,
-					  1,
-					  row,
-					  process_get_info_response,
-					  payload);
-		g_free(row);
+		search_soap_request(sipe_private,
+				    g_free,
+				    g_strdup(who),
+				    1,
+				    process_get_info_response,
+				    search_rows);
+		free_search_rows(search_rows);
 	}
 }
 
@@ -1686,6 +1777,15 @@ static void photo_response_data_free(struct photo_response_data *data)
 	g_free(data);
 }
 
+static void photo_response_data_remove(struct sipe_core_private *sipe_private,
+				       struct photo_response_data *data)
+{
+	data->request = NULL;
+	sipe_private->buddies->pending_photo_requests =
+		g_slist_remove(sipe_private->buddies->pending_photo_requests, data);
+	photo_response_data_free(data);
+}
+
 static void process_buddy_photo_response(struct sipe_core_private *sipe_private,
 					 guint status,
 					 GSList *headers,
@@ -1694,8 +1794,6 @@ static void process_buddy_photo_response(struct sipe_core_private *sipe_private,
 {
 	struct photo_response_data *rdata = (struct photo_response_data *) data;
 
-	rdata->request = NULL;
-
 	if (status == SIPE_HTTP_STATUS_OK) {
 		const gchar *len_str = sipe_utils_nameval_find(headers,
 							       "Content-Length");
@@ -1715,15 +1813,59 @@ static void process_buddy_photo_response(struct sipe_core_private *sipe_private,
 		}
 	}
 
-	sipe_private->buddies->pending_photo_requests =
-		g_slist_remove(sipe_private->buddies->pending_photo_requests, rdata);
+	photo_response_data_remove(sipe_private, rdata);
+}
 
-	photo_response_data_free(rdata);
+static void process_get_user_photo_response(struct sipe_core_private *sipe_private,
+					    guint status,
+					    SIPE_UNUSED_PARAMETER GSList *headers,
+					    const gchar *body,
+					    gpointer data)
+{
+	struct photo_response_data *rdata = (struct photo_response_data *) data;
+
+	if ((status == SIPE_HTTP_STATUS_OK) && body) {
+		sipe_xml *xml = sipe_xml_parse(body, strlen(body));
+		const sipe_xml *node = sipe_xml_child(xml,
+						      "Body/GetUserPhotoResponse/PictureData");
+
+		if (node) {
+			gchar *base64;
+			gsize photo_size;
+			guchar *photo;
+
+			/* decode photo data */
+			base64 = sipe_xml_data(node);
+			photo = g_base64_decode(base64, &photo_size);
+			g_free(base64);
+
+			/* EWS doesn't provide a hash -> calculate SHA-1 digest */
+			if (!rdata->photo_hash) {
+				guchar digest[SIPE_DIGEST_SHA1_LENGTH];
+				sipe_digest_sha1(photo, photo_size, digest);
+
+				/* rdata takes ownership of digest string */
+				rdata->photo_hash = buff_to_hex_str(digest,
+								    SIPE_DIGEST_SHA1_LENGTH);
+			}
+
+			/* backend frees "photo" */
+			sipe_backend_buddy_set_photo(SIPE_CORE_PUBLIC,
+						     rdata->who,
+						     photo,
+						     photo_size,
+						     rdata->photo_hash);
+		}
+
+		sipe_xml_free(xml);
+	}
+
+	photo_response_data_remove(sipe_private, rdata);
 }
 
 static gchar *create_x_ms_webticket_header(const gchar *wsse_security)
 {
-	gchar *assertion = sipe_xml_extract_raw(wsse_security, "saml:Assertion", TRUE);
+	gchar *assertion = sipe_xml_extract_raw(wsse_security, "Assertion", TRUE);
 	gchar *wsse_security_base64;
 	gchar *x_ms_webticket_header;
 
@@ -1742,6 +1884,118 @@ static gchar *create_x_ms_webticket_header(const gchar *wsse_security)
 	return x_ms_webticket_header;
 }
 
+/* see also sipe_ucs_http_request() */
+static struct sipe_http_request *get_user_photo_request(struct sipe_core_private *sipe_private,
+							struct photo_response_data *data,
+							const gchar *ews_url,
+							const gchar *email)
+{
+	gchar *soap = g_strdup_printf("<?xml version=\"1.0\"?>\r\n"
+				      "<soap:Envelope"
+				      " xmlns:m=\"http://schemas.microsoft.com/exchange/services/2006/messages\""
+				      " xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\""
+				      " xmlns:t=\"http://schemas.microsoft.com/exchange/services/2006/types\""
+				      " >"
+				      " <soap:Header>"
+				      "  <t:RequestServerVersion Version=\"Exchange2013\" />"
+				      " </soap:Header>"
+				      " <soap:Body>"
+				      "  <m:GetUserPhoto>"
+				      "   <m:Email>%s</m:Email>"
+				      "   <m:SizeRequested>HR48x48</m:SizeRequested>"
+				      "  </m:GetUserPhoto>"
+				      " </soap:Body>"
+				      "</soap:Envelope>",
+				      email);
+	struct sipe_http_request *request = sipe_http_request_post(sipe_private,
+								   ews_url,
+								   NULL,
+								   soap,
+								   "text/xml; charset=UTF-8",
+								   process_get_user_photo_response,
+								   data);
+	g_free(soap);
+
+	if (request) {
+		sipe_core_email_authentication(sipe_private,
+					       request);
+		sipe_http_request_allow_redirect(request);
+	} else {
+		SIPE_DEBUG_ERROR_NOFORMAT("get_user_photo_request: failed to create HTTP connection");
+	}
+
+	return(request);
+}
+
+static void photo_response_data_finalize(struct sipe_core_private *sipe_private,
+					 struct photo_response_data *data,
+					 const gchar *uri,
+					 const gchar *photo_hash)
+{
+	if (data->request) {
+		data->who        = g_strdup(uri);
+		data->photo_hash = g_strdup(photo_hash);
+
+		sipe_private->buddies->pending_photo_requests =
+			g_slist_append(sipe_private->buddies->pending_photo_requests, data);
+		sipe_http_request_ready(data->request);
+	} else {
+		photo_response_data_free(data);
+	}
+}
+
+void sipe_buddy_update_photo(struct sipe_core_private *sipe_private,
+			     const gchar *uri,
+			     const gchar *photo_hash,
+			     const gchar *photo_url,
+			     const gchar *headers)
+{
+	const gchar *photo_hash_old =
+		sipe_backend_buddy_get_photo_hash(SIPE_CORE_PUBLIC, uri);
+
+	if (!sipe_strequal(photo_hash, photo_hash_old)) {
+		struct photo_response_data *data = g_new0(struct photo_response_data, 1);
+
+		SIPE_DEBUG_INFO("sipe_buddy_update_photo: who '%s' url '%s' hash '%s'",
+				uri, photo_url, photo_hash);
+
+		/* Photo URL is embedded XML? */
+		if (g_str_has_prefix(photo_url, "<") &&
+		    g_str_has_suffix(photo_url, ">")) {
+			/* add dummy root to embedded XML string */
+			gchar *tmp = g_strdup_printf("<r>%s</r>", photo_url);
+			sipe_xml *xml = sipe_xml_parse(tmp, strlen(tmp));
+			g_free(tmp);
+
+			if (xml) {
+				gchar *ews_url = sipe_xml_data(sipe_xml_child(xml, "ewsUrl"));
+				gchar *email = sipe_xml_data(sipe_xml_child(xml, "primarySMTP"));
+
+				if (!is_empty(ews_url) && !is_empty(email))
+					data->request = get_user_photo_request(sipe_private,
+									       data,
+									       ews_url,
+									       email);
+
+				g_free(email);
+				g_free(ews_url);
+				sipe_xml_free(xml);
+			}
+		} else {
+			data->request = sipe_http_request_get(sipe_private,
+							      photo_url,
+							      headers,
+							      process_buddy_photo_response,
+							      data);
+		}
+
+		photo_response_data_finalize(sipe_private,
+					     data,
+					     uri,
+					     photo_hash);
+	}
+}
+
 static void get_photo_ab_entry_response(struct sipe_core_private *sipe_private,
 					const gchar *uri,
 					SIPE_UNUSED_PARAMETER const gchar *raw,
@@ -1751,8 +2005,6 @@ static void get_photo_ab_entry_response(struct sipe_core_private *sipe_private,
 	struct ms_dlx_data *mdd = callback_data;
 	gchar *photo_rel_path = NULL;
 	gchar *photo_hash = NULL;
-	const gchar *photo_hash_old =
-		sipe_backend_buddy_get_photo_hash(SIPE_CORE_PUBLIC, mdd->other);
 
 	if (soap_body) {
 		const sipe_xml *node;
@@ -1783,30 +2035,16 @@ static void get_photo_ab_entry_response(struct sipe_core_private *sipe_private,
 		}
 	}
 
-	if (sipe_private->addressbook_uri && photo_rel_path &&
-	    photo_hash && !sipe_strequal(photo_hash, photo_hash_old)) {
+	if (sipe_private->addressbook_uri && photo_rel_path && photo_hash) {
 		gchar *photo_url = g_strdup_printf("%s/%s",
 				sipe_private->addressbook_uri, photo_rel_path);
 		gchar *x_ms_webticket_header = create_x_ms_webticket_header(mdd->wsse_security);
 
-		struct photo_response_data *data = g_new(struct photo_response_data, 1);
-		data->who = g_strdup(mdd->other);
-		data->photo_hash = photo_hash;
-		photo_hash = NULL;
-
-		data->request = sipe_http_request_get(sipe_private,
-						      photo_url,
-						      x_ms_webticket_header,
-						      process_buddy_photo_response,
-						      data);
-
-		if (data->request) {
-			sipe_private->buddies->pending_photo_requests =
-				g_slist_append(sipe_private->buddies->pending_photo_requests, data);
-			sipe_http_request_ready(data->request);
-		} else {
-			photo_response_data_free(data);
-		}
+		sipe_buddy_update_photo(sipe_private,
+					mdd->other,
+					photo_hash,
+					photo_url,
+					x_ms_webticket_header);
 
 		g_free(x_ms_webticket_header);
 		g_free(photo_url);
@@ -1831,17 +2069,24 @@ static void buddy_fetch_photo(struct sipe_core_private *sipe_private,
 		/* Lync 2013 or newer: use UCS if contacts are migrated */
 		if (SIPE_CORE_PRIVATE_FLAG_IS(LYNC2013) &&
 		    sipe_ucs_is_migrated(sipe_private)) {
+			struct photo_response_data *data = g_new0(struct photo_response_data, 1);
 
-			sipe_ucs_get_photo(sipe_private, uri);
+			data->request = get_user_photo_request(sipe_private,
+							       data,
+							       sipe_ucs_ews_url(sipe_private),
+							       sipe_get_no_sip_uri(uri));
+			photo_response_data_finalize(sipe_private,
+						     data,
+						     uri,
+						     /* there is no hash */
+						     NULL);
 
 		/* Lync 2010: use [MS-DLX] */
 		} else if (sipe_private->dlx_uri         &&
 			   sipe_private->addressbook_uri) {
 			struct ms_dlx_data *mdd = g_new0(struct ms_dlx_data, 1);
 
-			mdd->search_rows = g_slist_append(mdd->search_rows, g_strdup("msRTCSIP-PrimaryUserAddress"));
-			mdd->search_rows = g_slist_append(mdd->search_rows, g_strdup(uri));
-
+			mdd->search_rows     = search_rows_for_uri(uri);
 			mdd->other           = g_strdup(uri);
 			mdd->max_returns     = 1;
 			mdd->callback        = get_photo_ab_entry_response;
