@@ -3,7 +3,7 @@
  *
  * pidgin-sipe
  *
- * Copyright (C) 2010-2013 SIPE Project <http://sipe.sourceforge.net/>
+ * Copyright (C) 2010-2016 SIPE Project <http://sipe.sourceforge.net/>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -542,24 +542,34 @@ static struct sipe_groupchat_msg *chatserver_command(struct sipe_core_private *s
 						     const gchar *cmd)
 {
 	struct sipe_groupchat *groupchat = sipe_private->groupchat;
-	struct sip_dialog *dialog = sipe_dialog_find(groupchat->session,
-						     groupchat->session->with);
 	struct sipe_groupchat_msg *msg = NULL;
 
-	if (dialog) {
-		struct transaction_payload *payload = g_new0(struct transaction_payload, 1);
-		struct transaction *trans;
+	if (groupchat->session) {
+		struct sip_dialog *dialog = sipe_dialog_find(groupchat->session,
+							     groupchat->session->with);
 
-		msg = generate_xccos_message(groupchat, cmd);
-		trans = sip_transport_info(sipe_private,
-					   "Content-Type: text/plain\r\n",
-					   msg->xccos,
-					   dialog,
-					   chatserver_command_response);
+		if (dialog) {
+			struct transaction *trans;
 
-		payload->destroy = sipe_groupchat_msg_remove;
-		payload->data    = msg;
-		trans->payload   = payload;
+			msg = generate_xccos_message(groupchat, cmd);
+			trans = sip_transport_info(sipe_private,
+						   "Content-Type: text/plain\r\n",
+						   msg->xccos,
+						   dialog,
+						   chatserver_command_response);
+
+			if (trans) {
+				struct transaction_payload *payload = g_new0(struct transaction_payload, 1);
+
+				payload->destroy = sipe_groupchat_msg_remove;
+				payload->data    = msg;
+				trans->payload   = payload;
+			} else {
+				/* SIP transport is no longer valid - give up */
+				sipe_groupchat_msg_remove(msg);
+				msg = NULL;
+			}
+		}
 	}
 
 	return(msg);
