@@ -3,7 +3,7 @@
 *
  * pidgin-sipe
  *
- * Copyright (C) 2013 SIPE Project <http://sipe.sourceforge.net/>
+ * Copyright (C) 2013-2015 SIPE Project <http://sipe.sourceforge.net/>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -39,6 +39,7 @@
 
 #include <glib.h>
 
+#include "sipe-common.h"
 #include "sipmsg.h"
 #include "sip-sec.h"
 #include "sipe-backend.h"
@@ -52,7 +53,7 @@
 #include "sipe-http-transport.h"
 
 struct sipe_http_session {
-	gchar *cookie; /* extremely simplistic cookie jar :-) */
+	GHashTable *cookie_jar;
 };
 
 struct sipe_http_request {
@@ -66,7 +67,6 @@ struct sipe_http_request {
 	gchar *content_type;   /* NULL if body == NULL */
 	gchar *authorization;
 
-	const gchar *domain;   /* not copied */
 	const gchar *user;     /* not copied */
 	const gchar *password; /* not copied */
 
@@ -100,6 +100,13 @@ static void sipe_http_request_free(struct sipe_core_private *sipe_private,
 	g_free(req);
 }
 
+static void add_cookie_cb(SIPE_UNUSED_PARAMETER const gchar *key,
+			  const gchar *cookie,
+			  GString *string)
+{
+	g_string_append_printf(string, "Cookie: %s\r\n", cookie);
+}
+
 static void sipe_http_request_send(struct sipe_http_connection_public *conn_public)
 {
 	struct sipe_http_request *req = conn_public->pending_requests->data;
@@ -113,8 +120,13 @@ static void sipe_http_request_send(struct sipe_http_connection_public *conn_publ
 					  strlen(req->body),
 					  req->content_type);
 
-	if (req->session && req->session->cookie)
-		cookie = g_strdup_printf("Cookie: %s\r\n", req->session->cookie);
+	if (req->session && g_hash_table_size(req->session->cookie_jar)) {
+		GString *cookies = g_string_new("");
+		g_hash_table_foreach(req->session->cookie_jar,
+				     (GHFunc) add_cookie_cb,
+				     cookies);
+		cookie = g_string_free(cookies, FALSE);
+	}
 
 	header = g_strdup_printf("%s /%s HTTP/1.1\r\n"
 				 "Host: %s\r\n"
@@ -332,7 +344,6 @@ static gboolean sipe_http_request_response_unauthorized(struct sipe_core_private
 			conn_public->context = sip_sec_create_context(type,
 								      !valid, /* Single Sign-On flag */
 								      TRUE,   /* connection-based for HTTP */
-								      valid ? req->domain   : NULL,
 								      valid ? req->user     : NULL,
 								      valid ? req->password : NULL);
 		}
@@ -405,39 +416,43 @@ static void sipe_http_request_response_callback(struct sipe_core_private *sipe_p
 						struct sipe_http_request *req,
 						struct sipmsg *msg)
 {
-	const gchar *hdr;
-
 	sipe_http_request_finalize_negotiate(req, msg);
 
 	/* Set-Cookie: RMID=732423sdfs73242; expires=Fri, 31-Dec-2010 23:59:59 GMT; path=/; domain=.example.net */
-	if (req->session &&
-	    ((hdr = sipmsg_find_header(msg, "Set-Cookie")) != NULL)) {
-		gchar **parts, **current;
-		const gchar *part;
-		gchar *new = NULL;
+	if (req->session) {
+		guint instance = 0;
+		const gchar *hdr;
 
-		g_free(req->session->cookie);
-		req->session->cookie = NULL;
+		/* extract all cookies from header */
+		while ((hdr = sipmsg_find_header_instance(msg,
+							  "Set-Cookie",
+							  instance++)) != NULL) {
+			gchar **parts, **current;
+			const gchar *part;
+			gchar *new = NULL;
 
-		current = parts = g_strsplit(hdr, ";", 0);
-		while ((part = *current++) != NULL) {
-			/* strip these parts from cookie */
-			if (!(strstr(part, "path=")    ||
-			      strstr(part, "domain=")  ||
-			      strstr(part, "expires=") ||
-			      strstr(part, "secure"))) {
-				gchar *tmp = new;
-				new = new ?
-					g_strconcat(new, ";", part, NULL) :
-					g_strdup(part);
-				g_free(tmp);
+			current = parts = g_strsplit(hdr, ";", 0);
+			while ((part = *current++) != NULL) {
+				/* strip these parts from cookie */
+				if (!(strstr(part, "path=")    ||
+				      strstr(part, "domain=")  ||
+				      strstr(part, "expires=") ||
+				      strstr(part, "secure"))) {
+					gchar *tmp = new;
+					new = new ?
+						g_strconcat(new, ";", part, NULL) :
+						g_strdup(part);
+					g_free(tmp);
+				}
 			}
-		}
-		g_strfreev(parts);
 
-		if (new) {
-			req->session->cookie = new;
-			SIPE_DEBUG_INFO("sipe_http_request_response_callback: cookie: %s", new);
+			if (new) {
+				g_hash_table_insert(req->session->cookie_jar,
+						    g_strdup(*parts),
+						    new);
+				SIPE_DEBUG_INFO("sipe_http_request_response_callback: cookie: %s", new);
+			}
+			g_strfreev(parts);
 		}
 	}
 
@@ -567,7 +582,6 @@ struct sipe_http_request *sipe_http_request_new(struct sipe_core_private *sipe_p
 	/* default authentication */
 	if (!SIPE_CORE_PRIVATE_FLAG_IS(SSO))
 		sipe_http_request_authentication(req,
-						 sipe_private->authdomain,
 						 sipe_private->authuser,
 						 sipe_private->password);
 
@@ -588,13 +602,18 @@ void sipe_http_request_ready(struct sipe_http_request *request)
 
 struct sipe_http_session *sipe_http_session_start(void)
 {
-	return(g_new0(struct sipe_http_session, 1));
+	struct sipe_http_session *session = g_new0(struct sipe_http_session, 1);
+	session->cookie_jar = g_hash_table_new_full(g_str_hash,
+						    g_str_equal,
+						    g_free,
+						    g_free);
+	return(session);
 }
 
 void sipe_http_session_close(struct sipe_http_session *session)
 {
 	if (session) {
-		g_free(session->cookie);
+		g_hash_table_destroy(session->cookie_jar);
 		g_free(session);
 	}
 }
@@ -625,12 +644,10 @@ void sipe_http_request_allow_redirect(struct sipe_http_request *request)
 }
 
 void sipe_http_request_authentication(struct sipe_http_request *request,
-				      const gchar *domain,
 				      const gchar *user,
 				      const gchar *password)
 {
 	request->flags   |= SIPE_HTTP_REQUEST_FLAG_AUTHDATA;
-	request->domain   = domain;
 	request->user     = user;
 	request->password = password;
 }
