@@ -3,7 +3,7 @@
  *
  * pidgin-sipe
  *
- * Copyright (C) 2011-2013 SIPE Project <http://sipe.sourceforge.net/>
+ * Copyright (C) 2011-2015 SIPE Project <http://sipe.sourceforge.net/>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -81,6 +81,11 @@ static void sipe_process_provisioning(struct sipe_core_private *sipe_private,
 static void sipe_process_provisioning_v2(struct sipe_core_private *sipe_private,
 					 struct sipmsg *msg)
 {
+#define READ_INT_FROM_NODE(node_name, field) { \
+	gchar *s = g_strstrip(sipe_xml_data(sipe_xml_child(node, node_name))); \
+	sipe_private->field = s ? atoi(s) : 0; \
+	g_free(s); }
+
 	sipe_xml *xn_provision_group_list;
 	const sipe_xml *node;
 
@@ -98,6 +103,8 @@ static void sipe_process_provisioning_v2(struct sipe_core_private *sipe_private,
 					"dlxExternalUrl" : "dlxInternalUrl";
 			const gchar *addressbook_uri_str = SIPE_CORE_PRIVATE_FLAG_IS(REMOTE_USER) ?
 					"absExternalServerUrl" : "absInternalServerUrl";
+			gchar *ucPC2PCAVEncryption = NULL;
+			gchar *ucPortRangeEnabled = NULL;
 
 			g_free(sipe_private->focus_factory_uri);
 			sipe_private->focus_factory_uri = sipe_xml_data(sipe_xml_child(node, "focusFactoryUri"));
@@ -129,6 +136,43 @@ static void sipe_process_provisioning_v2(struct sipe_core_private *sipe_private,
 					sipe_media_get_av_edge_credentials(sipe_private);
 #endif
 
+			ucPC2PCAVEncryption = g_strstrip(sipe_xml_data(sipe_xml_child(node, "ucPC2PCAVEncryption")));
+			if (sipe_strequal(ucPC2PCAVEncryption, "SupportEncryption")) {
+				sipe_private->server_av_encryption_policy = SIPE_ENCRYPTION_POLICY_OPTIONAL;
+			} else if (sipe_strequal(ucPC2PCAVEncryption, "DoNotSupportEncryption")) {
+				sipe_private->server_av_encryption_policy = SIPE_ENCRYPTION_POLICY_REJECTED;
+			} else {
+				// "RequireEncryption" or any unknown value.
+				sipe_private->server_av_encryption_policy = SIPE_ENCRYPTION_POLICY_REQUIRED;
+			}
+			g_free(ucPC2PCAVEncryption);
+
+			ucPortRangeEnabled = g_strstrip(sipe_xml_data(sipe_xml_child(node, "ucPortRangeEnabled")));
+			if (sipe_strequal(ucPortRangeEnabled, "true")) {
+				READ_INT_FROM_NODE("ucMinMediaPort", min_media_port)
+				READ_INT_FROM_NODE("ucMaxMediaPort", max_media_port)
+				READ_INT_FROM_NODE("ucMinAudioPort", min_audio_port)
+				READ_INT_FROM_NODE("ucMaxAudioPort", max_audio_port)
+				READ_INT_FROM_NODE("ucMinVideoPort", min_video_port)
+				READ_INT_FROM_NODE("ucMaxVideoPort", max_video_port)
+				READ_INT_FROM_NODE("ucMinAppSharingPort", min_appsharing_port)
+				READ_INT_FROM_NODE("ucMaxAppSharingPort", max_appsharing_port)
+				READ_INT_FROM_NODE("ucMinFileTransferPort", min_filetransfer_port)
+				READ_INT_FROM_NODE("ucMaxFileTransferPort", max_filetransfer_port)
+			} else {
+				sipe_private->min_media_port = 0;
+				sipe_private->max_media_port = 0;
+				sipe_private->min_audio_port = 0;
+				sipe_private->max_audio_port = 0;
+				sipe_private->min_video_port = 0;
+				sipe_private->max_video_port = 0;
+				sipe_private->min_appsharing_port = 0;
+				sipe_private->max_appsharing_port = 0;
+				sipe_private->min_filetransfer_port = 0;
+				sipe_private->max_filetransfer_port = 0;
+			}
+			g_free(ucPortRangeEnabled);
+
 		/* persistentChatConfiguration */
 		} else if (sipe_strequal("persistentChatConfiguration", node_name)) {
 			const sipe_xml *property;
@@ -171,6 +215,12 @@ static void sipe_process_provisioning_v2(struct sipe_core_private *sipe_private,
 		sipe_buddy_refresh_photos(sipe_private);
 	}
 
+	if (sipe_private->focus_factory_uri) {
+		/* Fill the list of conferencing capabilities enabled on
+		 * the server. */
+		sipe_conf_get_capabilities(sipe_private);
+	}
+
 	if (SIPE_CORE_PRIVATE_FLAG_IS(OCS2007))
 		/* persistentChatPool_uri has been set at this point */
 		sipe_groupchat_init(sipe_private);
@@ -672,6 +722,29 @@ static void process_incoming_notify_rlmi(struct sipe_core_private *sipe_private,
 						break;
 					}
 				}
+				/* photo */
+				for (node = sipe_xml_child(card, "photo");
+				     node;
+				     node = sipe_xml_twin(node)) {
+					gchar *photo_url = sipe_xml_data(sipe_xml_child(node, "uri"));
+					gchar *hash = sipe_xml_data(sipe_xml_child(node, "hash"));
+					gboolean found = FALSE;
+
+					if (!is_empty(uri) && !is_empty(hash)) {
+						sipe_buddy_update_photo(sipe_private,
+									uri,
+									hash,
+									photo_url,
+									NULL);
+						found = TRUE;
+					}
+
+					g_free(hash);
+					g_free(photo_url);
+
+					if (found)
+						break;
+				}
 			}
 		}
 		/* note */
