@@ -3,7 +3,7 @@
  *
  * pidgin-sipe
  *
- * Copyright (C) 2011-2013 SIPE Project <http://sipe.sourceforge.net/>
+ * Copyright (C) 2011-2015 SIPE Project <http://sipe.sourceforge.net/>
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -254,10 +254,9 @@ static gchar *extract_raw_xml_attribute(const gchar *xml,
 	return(data);
 }
 
-static gchar *generate_timestamp(const gchar *raw,
-				 const gchar *lifetime_tag)
+static gchar *generate_timestamp(const gchar *raw)
 {
-	gchar *lifetime = sipe_xml_extract_raw(raw, lifetime_tag, FALSE);
+	gchar *lifetime = sipe_xml_extract_raw(raw, "Lifetime", FALSE);
 	gchar *timestamp = NULL;
 	if (lifetime)
 		timestamp = g_strdup_printf("<wsu:Timestamp xmlns:wsu=\"http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd\" wsu:Id=\"timestamp\">%s</wsu:Timestamp>",
@@ -266,9 +265,19 @@ static gchar *generate_timestamp(const gchar *raw,
 	return(timestamp);
 }
 
+static gchar *generate_keydata(const gchar *raw)
+{
+	return(sipe_xml_extract_raw(raw, "Assertion", TRUE));
+}
+
+static gchar *generate_expires(const gchar *timestamp)
+{
+	return(sipe_xml_extract_raw(timestamp, "Expires", FALSE));
+}
+
 static gchar *generate_fedbearer_wsse(const gchar *raw)
 {
-	gchar *timestamp = generate_timestamp(raw, "wst:Lifetime");
+	gchar *timestamp = generate_timestamp(raw);
 	gchar *keydata   = sipe_xml_extract_raw(raw, "EncryptedData", TRUE);
 	gchar *wsse_security = NULL;
 
@@ -285,23 +294,15 @@ static gchar *generate_fedbearer_wsse(const gchar *raw)
 static void generate_federation_wsse(struct sipe_webticket *webticket,
 				     const gchar *raw)
 {
-	gchar *timestamp = generate_timestamp(raw, "t:Lifetime");
-	gchar *keydata   = sipe_xml_extract_raw(raw, "saml:Assertion", TRUE);
-
-	/* try alternative names */
-	if (!timestamp)
-		timestamp = generate_timestamp(raw, "wst:Lifetime");
-	if (!keydata)
-		keydata   = sipe_xml_extract_raw(raw, "saml1:Assertion", TRUE);
+	gchar *timestamp = generate_timestamp(raw);
+	gchar *keydata   = generate_keydata(raw);
 
 	/* clear old ADFS token */
 	g_free(webticket->adfs_token);
 	webticket->adfs_token = NULL;
 
 	if (timestamp && keydata) {
-		gchar *expires_string = sipe_xml_extract_raw(timestamp,
-							     "wsu:Expires",
-							     FALSE);
+		gchar *expires_string = generate_expires(timestamp);
 
 		if (expires_string) {
 
@@ -325,14 +326,12 @@ static gchar *generate_sha1_proof_wsse(const gchar *raw,
 				       struct sipe_tls_random *entropy,
 				       time_t *expires)
 {
-	gchar *timestamp = generate_timestamp(raw, "Lifetime");
-	gchar *keydata   = sipe_xml_extract_raw(raw, "saml:Assertion", TRUE);
+	gchar *timestamp = generate_timestamp(raw);
+	gchar *keydata   = generate_keydata(raw);
 	gchar *wsse_security = NULL;
 
 	if (timestamp && keydata) {
-		gchar *expires_string = sipe_xml_extract_raw(timestamp,
-							     "Expires",
-							     FALSE);
+		gchar *expires_string = generate_expires(timestamp);
 
 		if (entropy) {
 			gchar *assertionID = extract_raw_xml_attribute(keydata,
@@ -548,13 +547,27 @@ static void webticket_token(struct sipe_core_private *sipe_private,
 
 	} else if (uri) {
 		/* Retry with federated authentication? */
-		if (wcd->webticket_fedbearer_uri && !wcd->tried_fedbearer) {
-			SIPE_DEBUG_INFO("webticket_token: anonymous authentication to service %s failed, retrying with federated authentication",
-					uri);
+		if (wcd->webticket_fedbearer_uri) {
 
-			if (initiate_fedbearer(sipe_private, wcd)) {
-				/* callback data passed down the line */
-				wcd = NULL;
+			/* Authentication against ADFS failed? */
+			if (wcd->token_state == TOKEN_STATE_FEDERATION) {
+				struct sipe_webticket *webticket = sipe_private->webticket;
+
+				SIPE_DEBUG_INFO_NOFORMAT("webticket_token: ADFS authentication failed - assuming Multi-Factor Authentication (MFA)");
+
+				/* forget ADFS URI */
+				g_free(webticket->webticket_adfs_uri);
+				webticket->webticket_adfs_uri = NULL;
+			}
+
+			if (!wcd->tried_fedbearer) {
+				SIPE_DEBUG_INFO("webticket_token: anonymous authentication to service %s failed, retrying with federated authentication",
+						uri);
+
+				if (initiate_fedbearer(sipe_private, wcd)) {
+					/* callback data passed down the line */
+					wcd = NULL;
+				}
 			}
 		}
 	}
@@ -692,6 +705,7 @@ static gboolean initiate_fedbearer(struct sipe_core_private *sipe_private,
 
 	if (sipe_private->webticket->retrieved_realminfo) {
 		/* skip retrieval and go to authentication */
+		wcd->tried_fedbearer = TRUE;
 		success = fedbearer_authentication(sipe_private, wcd);
 	} else {
 		success = sipe_svc_realminfo(sipe_private,
@@ -700,8 +714,6 @@ static gboolean initiate_fedbearer(struct sipe_core_private *sipe_private,
 					     wcd);
 	}
 
-	wcd->tried_fedbearer = TRUE;
-
 	return(success);
 }
 
