@@ -3,7 +3,7 @@
  *
  * pidgin-sipe
  *
- * Copyright (C) 2010-2013 SIPE Project <http://sipe.sourceforge.net/>
+ * Copyright (C) 2010-2015 SIPE Project <http://sipe.sourceforge.net/>
  * Copyright (C) 2009 pier11 <pier11@operamail.com>
  *
  *
@@ -155,7 +155,7 @@
 
 #define SIPE_PURPLE_KEY_CHAT_SESSION "sipe"
 
-static struct sipe_chat_session *sipe_purple_chat_get_session(PurpleConversation *conv)
+struct sipe_chat_session *sipe_purple_chat_get_session(PurpleConversation *conv)
 {
 	return(
 #if PURPLE_VERSION_CHECK(3,0,0)
@@ -219,12 +219,21 @@ void sipe_purple_chat_leave(PurpleConnection *gc, int id)
 
 int sipe_purple_chat_send(PurpleConnection *gc,
 			  int id,
+#if PURPLE_VERSION_CHECK(3,0,0)
+			  PurpleMessage *msg)
+#else
 			  const char *what,
 			  SIPE_UNUSED_PARAMETER PurpleMessageFlags flags)
+#endif
 {
 	struct sipe_chat_session *session = sipe_purple_chat_find(gc, id);
 	if (!session) return -ENOTCONN;
-	sipe_core_chat_send(PURPLE_GC_TO_SIPE_CORE_PUBLIC, session, what);
+	sipe_core_chat_send(PURPLE_GC_TO_SIPE_CORE_PUBLIC, session,
+#if PURPLE_VERSION_CHECK(3,0,0)
+			purple_message_get_contents(msg));
+#else
+			what);
+#endif
 	return 1;
 }
 
@@ -259,6 +268,15 @@ static void sipe_purple_chat_menu_join_call_cb(SIPE_UNUSED_PARAMETER PurpleChat
 
 #endif
 
+static void sipe_purple_chat_menu_entry_info_cb(SIPE_UNUSED_PARAMETER PurpleChat *chat,
+						PurpleConversation *conv)
+{
+	gchar *tmp = sipe_core_conf_entry_info(PURPLE_CONV_TO_SIPE_CORE_PUBLIC,
+					       sipe_purple_chat_get_session(conv));
+	purple_notify_formatted(NULL, NULL, "", NULL, tmp, NULL, NULL);
+	g_free(tmp);
+}
+
 GList *
 sipe_purple_chat_menu(PurpleChat *chat)
 {
@@ -291,7 +309,7 @@ sipe_purple_chat_menu(PurpleChat *chat)
 		if (act)
 			menu = g_list_prepend(menu, act);
 #ifdef HAVE_VV
-		if (!sipe_core_media_in_call(PURPLE_CONV_TO_SIPE_CORE_PUBLIC)) {
+		if (!sipe_core_media_get_call(PURPLE_CONV_TO_SIPE_CORE_PUBLIC)) {
 			act = NULL;
 			act = purple_menu_action_new(_("Join conference call"),
 						     PURPLE_CALLBACK(sipe_purple_chat_menu_join_call_cb),
@@ -300,6 +318,10 @@ sipe_purple_chat_menu(PurpleChat *chat)
 				menu = g_list_prepend(menu, act);
 		}
 #endif
+		act = purple_menu_action_new(_("Meeting entry info"),
+					     PURPLE_CALLBACK(sipe_purple_chat_menu_entry_info_cb),
+					     conv, NULL);
+		menu = g_list_append(menu, act);
 	}
 
 	return menu;
@@ -349,13 +371,21 @@ struct sipe_backend_chat_session *sipe_backend_chat_create(struct sipe_core_publ
 {
 	struct sipe_backend_private *purple_private = sipe_public->backend_private;
 #if PURPLE_VERSION_CHECK(3,0,0)
-	PurpleChatConversation *conv =
+	PurpleChatConversation *conv;
 #else
-	PurpleConversation *conv =
+	PurpleConversation *conv;
 #endif
-		purple_serv_got_joined_chat(purple_private->gc,
-					    sipe_purple_chat_id(purple_private->gc),
-					    title);
+
+	/*
+	 * Adium calls back into SIPE code during execution of the following
+	 * libpurple API. That code needs access to "session". As "conv" is
+	 * still being initialized we can't use sipe_purple_chat_get_session().
+	 */
+	purple_private->adium_chat_session = session;
+	conv = purple_serv_got_joined_chat(purple_private->gc,
+					   sipe_purple_chat_id(purple_private->gc),
+					   title);
+	purple_private->adium_chat_session = NULL;
 #if PURPLE_VERSION_CHECK(3,0,0)
 	g_object_set_data(G_OBJECT(conv),
 #else
