@@ -3,7 +3,7 @@
  *
  * pidgin-sipe
  *
- * Copyright (C) 2010-2013 SIPE Project <http://sipe.sourceforge.net/>
+ * Copyright (C) 2010-2016 SIPE Project <http://sipe.sourceforge.net/>
  * Copyright (C) 2010 Jakub Adam <jakub.adam@ktknet.cz>
  * Copyright (C) 2010 Tomáš Hrabčík <tomas.hrabcik@tieto.com>
  *
@@ -30,11 +30,15 @@
 #include <fcntl.h>
 #include <errno.h>
 
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #include <glib.h>
 
 #include "version.h"
 #if PURPLE_VERSION_CHECK(3,0,0)
-#include "xfer.h"
+#include "protocol.h"
 #define PURPLE_XFER_TO_SIPE_CORE_PUBLIC        ((struct sipe_core_public *) purple_connection_get_protocol_data(purple_account_get_connection(purple_xfer_get_account(xfer))))
 #else
 #include "ft.h"
@@ -130,6 +134,20 @@ gssize sipe_backend_ft_write(struct sipe_file_transfer *ft,
 	return bytes_written;
 }
 
+static gboolean
+end_transfer_cb(gpointer data)
+{
+	purple_xfer_end((PurpleXfer *)data);
+	return FALSE;
+}
+
+void
+sipe_backend_ft_set_completed(struct sipe_file_transfer *ft)
+{
+	purple_xfer_set_completed(FT_TO_PURPLE_XFER, TRUE);
+	purple_timeout_add(0, end_transfer_cb, FT_TO_PURPLE_XFER);
+}
+
 void sipe_backend_ft_cancel_local(struct sipe_file_transfer *ft)
 {
 	purple_xfer_cancel_local(FT_TO_PURPLE_XFER);
@@ -143,104 +161,111 @@ void sipe_backend_ft_cancel_remote(struct sipe_file_transfer *ft)
 static void
 ft_free_xfer_struct(PurpleXfer *xfer)
 {
-	struct sipe_file_transfer *ft = PURPLE_XFER_TO_SIPE_FILE_TRANSFER;
-
-	if (ft) {
-		if (purple_xfer_get_watcher(xfer)) {
-			purple_input_remove(purple_xfer_get_watcher(xfer));
-			purple_xfer_set_watcher(xfer, 0);
-		}
-		sipe_core_ft_deallocate(ft);
-		purple_xfer_set_protocol_data(xfer, NULL);
+	if (purple_xfer_get_watcher(xfer)) {
+		purple_input_remove(purple_xfer_get_watcher(xfer));
+		purple_xfer_set_watcher(xfer, 0);
 	}
+
+	purple_xfer_set_protocol_data(xfer, NULL);
 }
 
 static void
 ft_request_denied(PurpleXfer *xfer)
 {
-	if (purple_xfer_get_xfer_type(xfer) == PURPLE_XFER_TYPE_RECEIVE)
-		sipe_core_ft_cancel(PURPLE_XFER_TO_SIPE_FILE_TRANSFER);
+	struct sipe_file_transfer *ft = PURPLE_XFER_TO_SIPE_FILE_TRANSFER;
+	if (ft->ft_request_denied) {
+		ft->ft_request_denied(ft);
+	}
+
 	ft_free_xfer_struct(xfer);
 }
 
 static void
-ft_incoming_init(PurpleXfer *xfer)
+ft_cancelled(PurpleXfer *xfer)
 {
-	sipe_core_ft_incoming_init(PURPLE_XFER_TO_SIPE_FILE_TRANSFER);
+	struct sipe_file_transfer *ft = PURPLE_XFER_TO_SIPE_FILE_TRANSFER;
+
+	if (ft->ft_cancelled &&
+	    purple_xfer_get_status(xfer) == PURPLE_XFER_STATUS_CANCEL_LOCAL) {
+		ft->ft_cancelled(ft);
+	}
+
+	ft_free_xfer_struct(xfer);
 }
 
 static void
-tftp_incoming_start(PurpleXfer *xfer)
+ft_init(PurpleXfer *xfer)
 {
-	sipe_core_tftp_incoming_start(PURPLE_XFER_TO_SIPE_FILE_TRANSFER,
-				      purple_xfer_get_size(xfer));
+	struct sipe_file_transfer *ft = PURPLE_XFER_TO_SIPE_FILE_TRANSFER;
+	g_return_if_fail(ft->ft_init);
+
+	ft->ft_init(ft,
+		    purple_xfer_get_filename(xfer),
+		    purple_xfer_get_size(xfer),
+		    purple_xfer_get_remote_user(xfer));
 }
 
 static void
-tftp_incoming_stop(PurpleXfer *xfer)
+ft_start(PurpleXfer *xfer)
 {
-	if (sipe_core_tftp_incoming_stop(PURPLE_XFER_TO_SIPE_FILE_TRANSFER)) {
+	struct sipe_file_transfer *ft = PURPLE_XFER_TO_SIPE_FILE_TRANSFER;
+
+	if (purple_xfer_get_xfer_type(xfer) == PURPLE_XFER_TYPE_RECEIVE) {
+		/* Set socket to non-blocking mode */
+		int flags = fcntl(purple_xfer_get_fd(xfer), F_GETFL, 0);
+		if (flags == -1) {
+			flags = 0;
+		}
+		/* @TODO: ignoring potential error return - how to handle? */
+		fcntl(purple_xfer_get_fd(xfer), F_SETFL, flags | O_NONBLOCK);
+	}
+
+	if (ft->ft_start) {
+		ft->ft_start(ft, purple_xfer_get_size(xfer));
+	}
+}
+
+static void
+ft_end(PurpleXfer *xfer)
+{
+	struct sipe_file_transfer *ft = PURPLE_XFER_TO_SIPE_FILE_TRANSFER;
+
+	if (!ft->ft_end || ft->ft_end(ft)) {
 		/* We're done with this transfer */
 		ft_free_xfer_struct(xfer);
-	} else {
+	} else if (purple_xfer_get_xfer_type(xfer) == PURPLE_XFER_TYPE_RECEIVE) {
+		/* Remove incomplete file from failed transfer. */
 		unlink(purple_xfer_get_local_filename(xfer));
 	}
 }
 
-static gssize tftp_read(guchar **buffer,
+static gssize
+ft_read(guchar **buffer,
 #if PURPLE_VERSION_CHECK(3,0,0)
 			size_t buffer_size,
 #endif
 			PurpleXfer *xfer)
 {
-	return sipe_core_tftp_read(PURPLE_XFER_TO_SIPE_FILE_TRANSFER,
-				   buffer,
-				   purple_xfer_get_bytes_remaining(xfer),
+	struct sipe_file_transfer *ft = PURPLE_XFER_TO_SIPE_FILE_TRANSFER;
+	g_return_val_if_fail(ft->ft_read, 0);
+	return ft->ft_read(ft, buffer, purple_xfer_get_bytes_remaining(xfer),
 #if PURPLE_VERSION_CHECK(3,0,0)
-				   buffer_size
+			   buffer_size
 #else
-				   xfer->current_buffer_size
+			   xfer->current_buffer_size
 #endif
-		);
-}
-
-static void
-ft_outgoing_init(PurpleXfer *xfer)
-{
-	sipe_core_ft_outgoing_init(PURPLE_XFER_TO_SIPE_FILE_TRANSFER,
-				   purple_xfer_get_filename(xfer),
-				   purple_xfer_get_size(xfer),
-				   purple_xfer_get_remote_user(xfer));
-}
-
-static void
-tftp_outgoing_start(PurpleXfer *xfer)
-{
-	/* Set socket to non-blocking mode */
-	int flags = fcntl(purple_xfer_get_fd(xfer), F_GETFL, 0);
-	if (flags == -1)
-		flags = 0;
-	/* @TODO: ignoring potential error return - how to handle? */
-	(void) fcntl(purple_xfer_get_fd(xfer), F_SETFL, flags | O_NONBLOCK);
-
-	sipe_core_tftp_outgoing_start(PURPLE_XFER_TO_SIPE_FILE_TRANSFER,
-				      purple_xfer_get_size(xfer));
-}
-
-static void
-tftp_outgoing_stop(PurpleXfer *xfer)
-{
-	if (sipe_core_tftp_outgoing_stop(PURPLE_XFER_TO_SIPE_FILE_TRANSFER)) {
-		/* We're done with this transfer */
-		ft_free_xfer_struct(xfer);
-	}
+	);
 }
 
 static gssize
-tftp_write(const guchar *buffer, size_t size, PurpleXfer *xfer)
+ft_write(const guchar *buffer, size_t size, PurpleXfer *xfer)
 {
-	gssize bytes_written = sipe_core_tftp_write(PURPLE_XFER_TO_SIPE_FILE_TRANSFER,
-						    buffer, size);
+	struct sipe_file_transfer *ft = PURPLE_XFER_TO_SIPE_FILE_TRANSFER;
+	gssize bytes_written = 0;
+
+	g_return_val_if_fail(ft->ft_write, 0);
+
+	bytes_written = ft->ft_write(ft, buffer, size);
 
 	if ((purple_xfer_get_bytes_remaining(xfer) - bytes_written) == 0)
 		purple_xfer_set_completed(xfer, TRUE);
@@ -248,7 +273,25 @@ tftp_write(const guchar *buffer, size_t size, PurpleXfer *xfer)
 	return bytes_written;
 }
 
-//******************************************************************************
+static PurpleXfer *
+create_xfer(PurpleAccount *account, PurpleXferType type, const char *who,
+	    struct sipe_file_transfer *ft)
+{
+	PurpleXfer *xfer = purple_xfer_new(account, type, who);
+	if (xfer) {
+		ft->backend_private = (struct sipe_backend_file_transfer *)xfer;
+
+		purple_xfer_set_protocol_data(xfer, ft);
+		purple_xfer_set_init_fnc(xfer, ft_init);
+		purple_xfer_set_request_denied_fnc(xfer, ft_request_denied);
+		purple_xfer_set_cancel_send_fnc(xfer, ft_cancelled);
+		purple_xfer_set_cancel_recv_fnc(xfer, ft_cancelled);
+		purple_xfer_set_start_fnc(xfer, ft_start);
+		purple_xfer_set_end_fnc(xfer, ft_end);
+	}
+
+	return xfer;
+}
 
 void sipe_backend_ft_incoming(struct sipe_core_public *sipe_public,
 			      struct sipe_file_transfer *ft,
@@ -257,31 +300,33 @@ void sipe_backend_ft_incoming(struct sipe_core_public *sipe_public,
 			      gsize file_size)
 {
 	struct sipe_backend_private *purple_private = sipe_public->backend_private;
-	PurpleXfer *xfer;
-
-	xfer = purple_xfer_new(purple_private->account,
-			       PURPLE_XFER_TYPE_RECEIVE,
-			       who);
-
+	PurpleXfer *xfer = create_xfer(purple_private->account,
+				       PURPLE_XFER_TYPE_RECEIVE, who, ft);
 	if (xfer) {
-		ft->backend_private = (struct sipe_backend_file_transfer *)xfer;
-		purple_xfer_set_protocol_data(xfer, ft);
-
 		purple_xfer_set_filename(xfer, file_name);
 		purple_xfer_set_size(xfer, file_size);
 
-		purple_xfer_set_init_fnc(xfer, ft_incoming_init);
-		purple_xfer_set_request_denied_fnc(xfer, ft_request_denied);
-		purple_xfer_set_cancel_send_fnc(xfer, ft_free_xfer_struct);
-		purple_xfer_set_cancel_recv_fnc(xfer, ft_free_xfer_struct);
-		purple_xfer_set_start_fnc(xfer, tftp_incoming_start);
-		purple_xfer_set_end_fnc(xfer, tftp_incoming_stop);
-		purple_xfer_set_read_fnc(xfer, tftp_read);
-
 		purple_xfer_request(xfer);
 	}
 }
 
+void
+sipe_backend_ft_outgoing(struct sipe_core_public *sipe_public,
+			 struct sipe_file_transfer *ft,
+			 const gchar *who,
+			 const gchar *file_name)
+{
+	struct sipe_backend_private *purple_private = sipe_public->backend_private;
+	PurpleXfer *xfer = create_xfer(purple_private->account,
+				       PURPLE_XFER_TYPE_SEND, who, ft);
+	if (xfer) {
+		if (file_name != NULL)
+			purple_xfer_request_accepted(xfer, file_name);
+		else
+			purple_xfer_request(xfer);
+	}
+}
+
 static void
 connect_cb(gpointer data, gint fd, SIPE_UNUSED_PARAMETER const gchar *error_message)
 {
@@ -299,6 +344,13 @@ void
 sipe_backend_ft_start(struct sipe_file_transfer *ft, struct sipe_backend_fd *fd,
 		      const char* ip, unsigned port)
 {
+	PurpleXferType type = purple_xfer_get_xfer_type(FT_TO_PURPLE_XFER);
+	if (type == PURPLE_XFER_TYPE_SEND && ft->ft_write) {
+		purple_xfer_set_write_fnc(FT_TO_PURPLE_XFER, ft_write);
+	} else if (type == PURPLE_XFER_TYPE_RECEIVE && ft->ft_read) {
+		purple_xfer_set_read_fnc(FT_TO_PURPLE_XFER, ft_read);
+	}
+
 	if (ip && port && !sipe_backend_ft_is_incoming(ft)) {
 		/* Purple accepts ip & port only for incoming file transfers.
 		 * If we want to send file with Sender-Connect = TRUE negotiated,
@@ -320,42 +372,7 @@ void sipe_purple_ft_send_file(PurpleConnection *gc,
 			      const char *who,
 			      const char *file)
 {
-	PurpleXfer *xfer = sipe_purple_ft_new_xfer(gc, who);
-
-	if (xfer) {
-		if (file != NULL)
-			purple_xfer_request_accepted(xfer, file);
-		else
-			purple_xfer_request(xfer);
-	}
-}
-
-PurpleXfer *sipe_purple_ft_new_xfer(PurpleConnection *gc, const char *who)
-{
-	PurpleXfer *xfer = NULL;
-
-	if (PURPLE_CONNECTION_IS_VALID(gc)) {
-		xfer = purple_xfer_new(purple_connection_get_account(gc),
-				       PURPLE_XFER_TYPE_SEND,
-				       who);
-
-		if (xfer) {
-			struct sipe_file_transfer *ft = sipe_core_ft_allocate(PURPLE_GC_TO_SIPE_CORE_PUBLIC);
-
-			ft->backend_private = (struct sipe_backend_file_transfer *)xfer;
-			purple_xfer_set_protocol_data(xfer, ft);
-
-			purple_xfer_set_init_fnc(xfer, ft_outgoing_init);
-			purple_xfer_set_request_denied_fnc(xfer, ft_request_denied);
-			purple_xfer_set_cancel_send_fnc(xfer, ft_free_xfer_struct);
-			purple_xfer_set_cancel_recv_fnc(xfer, ft_free_xfer_struct);
-			purple_xfer_set_start_fnc(xfer, tftp_outgoing_start);
-			purple_xfer_set_end_fnc(xfer, tftp_outgoing_stop);
-			purple_xfer_set_write_fnc(xfer, tftp_write);
-		}
-	}
-
-	return xfer;
+	sipe_core_ft_create_outgoing(PURPLE_GC_TO_SIPE_CORE_PUBLIC, who, file);
 }
 
 gboolean
