@@ -3,7 +3,7 @@
  *
  * pidgin-sipe
  *
- * Copyright (C) 2011-2013 SIPE Project <http://sipe.sourceforge.net/>
+ * Copyright (C) 2011-2015 SIPE Project <http://sipe.sourceforge.net/>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,9 +24,12 @@
 #include "config.h"
 #endif
 
+#include <string.h>
+
 #include <glib.h>
 
 #include "notify.h"
+#include "request.h"
 
 #include "version.h"
 #if PURPLE_VERSION_CHECK(3,0,0)
@@ -151,6 +154,98 @@ void sipe_backend_search_results_finalize(struct sipe_core_public *sipe_public,
 
 }
 
+static void sipe_purple_find_contact_cb(PurpleConnection *gc,
+					PurpleRequestFields *fields)
+{
+	GList *entries = purple_request_field_group_get_fields(purple_request_fields_get_groups(fields)->data);
+	const gchar *given_name = NULL;
+	const gchar *surname    = NULL;
+	const gchar *email      = NULL;
+	const gchar *sipid      = NULL;
+	const gchar *company    = NULL;
+	const gchar *country    = NULL;
+
+	while (entries) {
+		PurpleRequestField *field = entries->data;
+		const char *id = purple_request_field_get_id(field);
+		const char *value = purple_request_field_string_get_value(field);
+
+		SIPE_DEBUG_INFO("sipe_purple_find_contact_cb: %s = '%s'", id, value ? value : "");
+
+		if (value && strlen(value)) {
+			if (strcmp(id, "given") == 0) {
+				given_name = value;
+			} else if (strcmp(id, "surname") == 0) {
+				surname = value;
+			} else if (strcmp(id, "email") == 0) {
+				email = value;
+			} else if (strcmp(id, "sipid") == 0) {
+				sipid = value;
+			} else if (strcmp(id, "company") == 0) {
+				company = value;
+			} else if (strcmp(id, "country") == 0) {
+				country = value;
+			}
+		}
+
+		entries = g_list_next(entries);
+	};
+
+	sipe_core_buddy_search(PURPLE_GC_TO_SIPE_CORE_PUBLIC,
+			       NULL,
+			       given_name,
+			       surname,
+			       email,
+			       sipid,
+			       company,
+			       country);
+}
+
+#if PURPLE_VERSION_CHECK(3,0,0)
+void sipe_purple_show_find_contact(PurpleProtocolAction *action)
+{
+	PurpleConnection *gc = action->connection;
+#else
+void sipe_purple_show_find_contact(PurplePluginAction *action)
+{
+	PurpleConnection *gc = (PurpleConnection *) action->context;
+#endif
+	PurpleRequestFields *fields;
+	PurpleRequestFieldGroup *group;
+	PurpleRequestField *field;
+
+	fields = purple_request_fields_new();
+	group = purple_request_field_group_new(NULL);
+	purple_request_fields_add_group(fields, group);
+
+	field = purple_request_field_string_new("given", _("First name"), NULL, FALSE);
+	purple_request_field_group_add_field(group, field);
+	field = purple_request_field_string_new("surname", _("Last name"), NULL, FALSE);
+	purple_request_field_group_add_field(group, field);
+	field = purple_request_field_string_new("email", _("Email"), NULL, FALSE);
+	purple_request_field_group_add_field(group, field);
+	field = purple_request_field_string_new("sipid", _("SIP ID"), NULL, FALSE);
+	purple_request_field_group_add_field(group, field);
+	field = purple_request_field_string_new("company", _("Company"), NULL, FALSE);
+	purple_request_field_group_add_field(group, field);
+	field = purple_request_field_string_new("country", _("Country"), NULL, FALSE);
+	purple_request_field_group_add_field(group, field);
+
+	purple_request_fields(gc,
+			      _("Search"),
+			      _("Search for a contact"),
+			      _("Enter the information for the person you wish to find. Empty fields will be ignored."),
+			      fields,
+			      _("_Search"), G_CALLBACK(sipe_purple_find_contact_cb),
+			      _("_Cancel"), NULL,
+#if PURPLE_VERSION_CHECK(3,0,0)
+			      purple_request_cpar_from_connection(gc),
+#else
+			      purple_connection_get_account(gc), NULL, NULL,
+#endif
+			      gc);
+}
+
 /*
   Local Variables:
   mode: c
