@@ -3,7 +3,7 @@
  *
  * pidgin-sipe
  *
- * Copyright (C) 2012-2013 SIPE Project <http://sipe.sourceforge.net/>
+ * Copyright (C) 2012-2016 SIPE Project <http://sipe.sourceforge.net/>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -125,7 +125,15 @@ void sipe_backend_chat_topic(SIPE_UNUSED_PARAMETER struct sipe_backend_chat_sess
 
 void sipe_backend_ft_error(SIPE_UNUSED_PARAMETER struct sipe_file_transfer *ft,
 			   SIPE_UNUSED_PARAMETER const gchar *errmsg) {}
+const gchar *sipe_backend_ft_get_error(SIPE_UNUSED_PARAMETER struct sipe_file_transfer *ft) { return(""); }
 void sipe_backend_ft_deallocate(SIPE_UNUSED_PARAMETER struct sipe_file_transfer *ft) {}
+gssize sipe_backend_ft_read(SIPE_UNUSED_PARAMETER struct sipe_file_transfer *ft,
+			    SIPE_UNUSED_PARAMETER guchar *data,
+			    SIPE_UNUSED_PARAMETER gsize size) { return(-1); }
+gssize sipe_backend_ft_write(SIPE_UNUSED_PARAMETER struct sipe_file_transfer *ft,
+			     SIPE_UNUSED_PARAMETER const guchar *data,
+			     SIPE_UNUSED_PARAMETER gsize size) { return(-1); }
+void sipe_backend_ft_set_completed(SIPE_UNUSED_PARAMETER struct sipe_file_transfer *ft) {}
 void sipe_backend_ft_cancel_local(SIPE_UNUSED_PARAMETER struct sipe_file_transfer *ft) {}
 void sipe_backend_ft_cancel_remote(SIPE_UNUSED_PARAMETER struct sipe_file_transfer *ft) {}
 void sipe_backend_ft_incoming(SIPE_UNUSED_PARAMETER struct sipe_core_public *sipe_public,
@@ -133,6 +141,10 @@ void sipe_backend_ft_incoming(SIPE_UNUSED_PARAMETER struct sipe_core_public *sip
 			      SIPE_UNUSED_PARAMETER const gchar *who,
 			      SIPE_UNUSED_PARAMETER const gchar *file_name,
 			      SIPE_UNUSED_PARAMETER gsize file_size) {}
+void sipe_backend_ft_outgoing(SIPE_UNUSED_PARAMETER struct sipe_core_public *sipe_public,
+			      SIPE_UNUSED_PARAMETER struct sipe_file_transfer *ft,
+			      SIPE_UNUSED_PARAMETER const gchar *who,
+			      SIPE_UNUSED_PARAMETER const gchar *file_name) {}
 void sipe_backend_ft_start(SIPE_UNUSED_PARAMETER struct sipe_file_transfer *ft,
 			   SIPE_UNUSED_PARAMETER struct sipe_backend_fd *fd,
 			   SIPE_UNUSED_PARAMETER const char* ip,
@@ -169,7 +181,7 @@ gchar *sipe_backend_markup_strip_html(SIPE_UNUSED_PARAMETER SIPE_UNUSED_PARAMETE
 struct sipe_backend_media *sipe_backend_media_new(SIPE_UNUSED_PARAMETER struct sipe_core_public *sipe_public,
 						  SIPE_UNUSED_PARAMETER struct sipe_media_call *call,
 						  SIPE_UNUSED_PARAMETER const gchar *participant,
-						  SIPE_UNUSED_PARAMETER gboolean initiator) { return(NULL); }
+						  SIPE_UNUSED_PARAMETER SipeMediaCallFlags flags) { return(NULL); }
 void sipe_backend_media_free(SIPE_UNUSED_PARAMETER struct sipe_backend_media *media) {}
 void sipe_backend_media_set_cname(SIPE_UNUSED_PARAMETER struct sipe_backend_media *media,
 				  SIPE_UNUSED_PARAMETER gchar *cname) {}
@@ -177,38 +189,39 @@ struct sipe_backend_media_relays * sipe_backend_media_relays_convert(SIPE_UNUSED
 								     SIPE_UNUSED_PARAMETER gchar *username,
 								     SIPE_UNUSED_PARAMETER gchar *password) { return(NULL); }
 void sipe_backend_media_relays_free(SIPE_UNUSED_PARAMETER struct sipe_backend_media_relays *media_relays) {}
-struct sipe_backend_stream *sipe_backend_media_add_stream(SIPE_UNUSED_PARAMETER struct sipe_backend_media *media,
-							  SIPE_UNUSED_PARAMETER const gchar *id,
-							  SIPE_UNUSED_PARAMETER const gchar *participant,
-							  SIPE_UNUSED_PARAMETER SipeMediaType type,
-							  SIPE_UNUSED_PARAMETER SipeIceVersion ice_version,
-							  SIPE_UNUSED_PARAMETER gboolean initiator,
-							  SIPE_UNUSED_PARAMETER struct sipe_backend_media_relays *media_relays) { return(NULL); }
-void sipe_backend_media_remove_stream(SIPE_UNUSED_PARAMETER struct sipe_backend_media *media,
-				      SIPE_UNUSED_PARAMETER struct sipe_backend_stream *stream) {}
-GSList *sipe_backend_media_get_streams(SIPE_UNUSED_PARAMETER struct sipe_backend_media *media) { return(NULL); }
-struct sipe_backend_stream *sipe_backend_media_get_stream_by_id(SIPE_UNUSED_PARAMETER struct sipe_backend_media *media,
-								SIPE_UNUSED_PARAMETER const gchar *id) { return(NULL); }
-void sipe_backend_media_add_remote_candidates(SIPE_UNUSED_PARAMETER struct sipe_backend_media *media,
-					      SIPE_UNUSED_PARAMETER struct sipe_backend_stream *stream,
+struct sipe_backend_media_stream *sipe_backend_media_add_stream(SIPE_UNUSED_PARAMETER struct sipe_media_call *media,
+								SIPE_UNUSED_PARAMETER const gchar *id,
+								SIPE_UNUSED_PARAMETER const gchar *participant,
+								SIPE_UNUSED_PARAMETER SipeMediaType type,
+								SIPE_UNUSED_PARAMETER SipeIceVersion ice_version,
+								SIPE_UNUSED_PARAMETER gboolean initiator,
+								SIPE_UNUSED_PARAMETER struct sipe_backend_media_relays *media_relays,
+								SIPE_UNUSED_PARAMETER guint min_port,
+								SIPE_UNUSED_PARAMETER guint max_port) { return(NULL); }
+void sipe_backend_media_add_remote_candidates(SIPE_UNUSED_PARAMETER struct sipe_media_call *media,
+					      SIPE_UNUSED_PARAMETER struct sipe_media_stream *stream,
 					      SIPE_UNUSED_PARAMETER GList *candidates) {}
-gboolean sipe_backend_media_is_initiator(SIPE_UNUSED_PARAMETER struct sipe_backend_media *media,
-					 SIPE_UNUSED_PARAMETER struct sipe_backend_stream *stream) { return(FALSE); }
+gboolean sipe_backend_media_is_initiator(SIPE_UNUSED_PARAMETER struct sipe_media_call *media,
+					 SIPE_UNUSED_PARAMETER struct sipe_media_stream *stream) { return(FALSE); }
 gboolean sipe_backend_media_accepted(SIPE_UNUSED_PARAMETER struct sipe_backend_media *media) { return(FALSE); }
-gboolean sipe_backend_stream_initialized(SIPE_UNUSED_PARAMETER struct sipe_backend_media *media,
-					 SIPE_UNUSED_PARAMETER struct sipe_backend_stream *stream) { return(FALSE); }
-GList *sipe_backend_media_get_active_local_candidates(SIPE_UNUSED_PARAMETER struct sipe_backend_media *media,
-						      SIPE_UNUSED_PARAMETER struct sipe_backend_stream *stream) { return(NULL); }
-GList *sipe_backend_media_get_active_remote_candidates(SIPE_UNUSED_PARAMETER struct sipe_backend_media *media,
-						       SIPE_UNUSED_PARAMETER struct sipe_backend_stream *stream) { return(NULL); }
-const gchar *sipe_backend_stream_get_id(SIPE_UNUSED_PARAMETER struct sipe_backend_stream *stream) { return(""); }
-void sipe_backend_stream_hold(SIPE_UNUSED_PARAMETER struct sipe_backend_media *media,
-			      SIPE_UNUSED_PARAMETER struct sipe_backend_stream *stream,
+gboolean sipe_backend_stream_initialized(SIPE_UNUSED_PARAMETER struct sipe_media_call *media,
+					 SIPE_UNUSED_PARAMETER struct sipe_media_stream *stream) { return(FALSE); }
+GList *sipe_backend_media_stream_get_active_local_candidates(SIPE_UNUSED_PARAMETER struct sipe_media_stream *stream) { return(NULL); }
+GList *sipe_backend_media_stream_get_active_remote_candidates(SIPE_UNUSED_PARAMETER struct sipe_media_stream *stream) { return(NULL); }
+void sipe_backend_media_set_encryption_keys(SIPE_UNUSED_PARAMETER struct sipe_media_call *media,
+					    SIPE_UNUSED_PARAMETER struct sipe_media_stream *stream,
+					    SIPE_UNUSED_PARAMETER const guchar *encryption_key,
+					    SIPE_UNUSED_PARAMETER const guchar *decryption_key) {}
+void sipe_backend_stream_hold(SIPE_UNUSED_PARAMETER struct sipe_media_call *media,
+			      SIPE_UNUSED_PARAMETER struct sipe_media_stream *stream,
 			      SIPE_UNUSED_PARAMETER gboolean local) {}
-void sipe_backend_stream_unhold(SIPE_UNUSED_PARAMETER struct sipe_backend_media *media,
-				SIPE_UNUSED_PARAMETER struct sipe_backend_stream *stream,
+void sipe_backend_stream_unhold(SIPE_UNUSED_PARAMETER struct sipe_media_call *media,
+				SIPE_UNUSED_PARAMETER struct sipe_media_stream *stream,
 				SIPE_UNUSED_PARAMETER gboolean local) {}
-gboolean sipe_backend_stream_is_held(SIPE_UNUSED_PARAMETER struct sipe_backend_stream *stream) { return(FALSE); }
+gboolean sipe_backend_stream_is_held(SIPE_UNUSED_PARAMETER struct sipe_media_stream *stream) { return(FALSE); }
+void sipe_backend_media_stream_end(SIPE_UNUSED_PARAMETER struct sipe_media_call *media,
+				   SIPE_UNUSED_PARAMETER struct sipe_media_stream *stream) {}
+void sipe_backend_media_stream_free(SIPE_UNUSED_PARAMETER struct sipe_backend_media_stream *stream) {}
 struct sipe_backend_codec *sipe_backend_codec_new(SIPE_UNUSED_PARAMETER int id,
 						  SIPE_UNUSED_PARAMETER const char *name,
 						  SIPE_UNUSED_PARAMETER SipeMediaType type,
@@ -221,11 +234,11 @@ void sipe_backend_codec_add_optional_parameter(SIPE_UNUSED_PARAMETER struct sipe
 					       SIPE_UNUSED_PARAMETER const gchar *name,
 					       SIPE_UNUSED_PARAMETER const gchar *value) {}
 GList *sipe_backend_codec_get_optional_parameters(SIPE_UNUSED_PARAMETER struct sipe_backend_codec *codec) { return(NULL); }
-gboolean sipe_backend_set_remote_codecs(SIPE_UNUSED_PARAMETER struct sipe_backend_media *media,
-					SIPE_UNUSED_PARAMETER struct sipe_backend_stream *stream,
+gboolean sipe_backend_set_remote_codecs(SIPE_UNUSED_PARAMETER struct sipe_media_call *media,
+					SIPE_UNUSED_PARAMETER struct sipe_media_stream *stream,
 					SIPE_UNUSED_PARAMETER GList *codecs) { return(FALSE); }
-GList* sipe_backend_get_local_codecs(SIPE_UNUSED_PARAMETER struct sipe_backend_media *media,
-				     SIPE_UNUSED_PARAMETER struct sipe_backend_stream *stream) { return(NULL); }
+GList* sipe_backend_get_local_codecs(SIPE_UNUSED_PARAMETER struct sipe_media_call *media,
+				     SIPE_UNUSED_PARAMETER struct sipe_media_stream *stream) { return(NULL); }
 struct sipe_backend_candidate * sipe_backend_candidate_new(SIPE_UNUSED_PARAMETER const gchar *foundation,
 							   SIPE_UNUSED_PARAMETER SipeComponentType component,
 							   SIPE_UNUSED_PARAMETER SipeCandidateType type,
@@ -248,14 +261,21 @@ void sipe_backend_candidate_set_priority(SIPE_UNUSED_PARAMETER struct sipe_backe
 SipeComponentType sipe_backend_candidate_get_component_type(SIPE_UNUSED_PARAMETER struct sipe_backend_candidate *candidate) { return(SIPE_COMPONENT_NONE); }
 SipeCandidateType sipe_backend_candidate_get_type(SIPE_UNUSED_PARAMETER struct sipe_backend_candidate *candidate) { return(SIPE_CANDIDATE_TYPE_ANY); }
 SipeNetworkProtocol sipe_backend_candidate_get_protocol(SIPE_UNUSED_PARAMETER struct sipe_backend_candidate *candidate) { return(SIPE_NETWORK_PROTOCOL_TCP_ACTIVE); }
-GList* sipe_backend_get_local_candidates(SIPE_UNUSED_PARAMETER struct sipe_backend_media *media,
-					 SIPE_UNUSED_PARAMETER struct sipe_backend_stream *stream) { return(NULL); }
+GList* sipe_backend_get_local_candidates(SIPE_UNUSED_PARAMETER struct sipe_media_call *media,
+					 SIPE_UNUSED_PARAMETER struct sipe_media_stream *stream) { return(NULL); }
 void sipe_backend_media_accept(SIPE_UNUSED_PARAMETER struct sipe_backend_media *media,
 			       SIPE_UNUSED_PARAMETER gboolean local) {}
 void sipe_backend_media_hangup(SIPE_UNUSED_PARAMETER struct sipe_backend_media *media,
 			       SIPE_UNUSED_PARAMETER gboolean local) {}
 void sipe_backend_media_reject(SIPE_UNUSED_PARAMETER struct sipe_backend_media *media,
 			       SIPE_UNUSED_PARAMETER gboolean local) {}
+SipeEncryptionPolicy sipe_backend_media_get_encryption_policy(SIPE_UNUSED_PARAMETER struct sipe_core_public *sipe_public) { return(SIPE_ENCRYPTION_POLICY_REJECTED); }
+gssize sipe_backend_media_stream_read(SIPE_UNUSED_PARAMETER struct sipe_media_stream *stream,
+				      SIPE_UNUSED_PARAMETER guint8 *buffer,
+				      SIPE_UNUSED_PARAMETER gsize len) { return(-1); }
+gssize sipe_backend_media_stream_write(SIPE_UNUSED_PARAMETER struct sipe_media_stream *stream,
+				       SIPE_UNUSED_PARAMETER guint8 *buffer,
+				       SIPE_UNUSED_PARAMETER gsize len) { return(-1); }
 #endif
 
 /** NETWORK ******************************************************************/
@@ -267,6 +287,7 @@ struct sipe_backend_listendata *sipe_backend_network_listen_range(SIPE_UNUSED_PA
 								  SIPE_UNUSED_PARAMETER gpointer data) { return(NULL); }
 void sipe_backend_network_listen_cancel(SIPE_UNUSED_PARAMETER struct sipe_backend_listendata *ldata) {}
 
+struct sipe_backend_fd *sipe_backend_fd_from_int(SIPE_UNUSED_PARAMETER int fd) { return (NULL); }
 gboolean sipe_backend_fd_is_valid(SIPE_UNUSED_PARAMETER struct sipe_backend_fd *fd) { return(FALSE); }
 void sipe_backend_fd_free(SIPE_UNUSED_PARAMETER struct sipe_backend_fd *fd) {}
 
